@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2006-2016 Yves Renard, Julien Pommier, Jeremie Lasry.
+ Copyright (C) 2006-2017 Yves Renard, Julien Pommier, Jeremie Lasry.
 
  This file is a part of GetFEM++
 
@@ -140,7 +140,8 @@ bool bilaplacian_crack_problem::solve_moment(plain_vector &U) {
 	 if (gmm::sqr(mesh.points_of_convex(i)[j][0]) + 
 	     gmm::sqr(mesh.points_of_convex(i)[j][1]) > 
 	     gmm::sqr(enr_area_radius)) 
-	   pm_convexes.sup(i); break;
+	   pm_convexes.sup(i);
+	 break;
        }
      }
       
