@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2006-2016 Yves Renard, Julien Pommier, Jeremie Lasry.
+ Copyright (C) 2006-2017 Yves Renard, Julien Pommier, Jeremie Lasry.
 
  This file is a part of GetFEM++
 
@@ -28,6 +28,8 @@
 
 using std::endl; using std::cout; using std::cerr;
 using std::ends; using std::cin;
+template <typename T> std::ostream &operator <<
+  (std::ostream &o, const std::vector<T>& m) { gmm::write(o,m); return o; }
 
 size_type is_global_dof_type_bis(getfem::pdof_description dof){
 size_type global_dof = 0 ;
@@ -600,7 +602,8 @@ void bilaplacian_crack_problem::compute_error(plain_vector &U) {
 	  if (gmm::sqr(mesh.points_of_convex(cv)[j][0] ) + 
 	      gmm::sqr(mesh.points_of_convex(cv)[j][1] ) > 
 	      gmm::sqr(radius_split_domain)) 
-	    in_area = false; break; 
+	    in_area = false;
+	  break; 
 	}
 	  if (in_area) r_center.add(cv) ;
 	  else r_ext.add(cv) ;
@@ -740,7 +743,8 @@ bool bilaplacian_crack_problem::solve(plain_vector &U) {
 	 if (gmm::sqr(mesh.points_of_convex(i)[j][0]) + 
 	     gmm::sqr(mesh.points_of_convex(i)[j][1]) > 
 	     gmm::sqr(enr_area_radius)) 
-	   pm_convexes.sup(i); break;
+	   pm_convexes.sup(i);
+	 break;
        }
      }
       
