@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2006-2016 Yves Renard, Julien Pommier, Jeremie Lasry.
+ Copyright (C) 2006-2017 Yves Renard, Julien Pommier, Jeremie Lasry.
 
  This file is a part of GetFEM++
 
@@ -79,9 +79,11 @@ void asm_normal_derivative_dirichlet_constraints_bis
     if (!R_must_be_derivated) {
       asm_normal_source_term(R, mim, mf_mult, mf_r, r_data, rg);
     } else {
-      asm_real_or_complex_1_param
-        (R, mim, mf_mult, mf_r, r_data, rg,
-         "R=data(#2); V(#1)+=comp(Grad(#1).Normal().Grad(#2).Normal())(i,j,k,k).R(j)");
+      asm_real_or_complex_1_param_vec
+	(R, mim, mf_mult, &mf_r, r_data, rg, "(Grad_A.Normal)*(Grad_Test_u.Normal)");
+      // asm_real_or_complex_1_param
+      //   (R, mim, mf_mult, mf_r, r_data, rg,
+      //    "R=data(#2); V(#1)+=comp(Grad(#1).Normal().Grad(#2).Normal())(:,i,i,j,k,k).R(j)");
     }
   }
 }
@@ -1089,14 +1091,14 @@ void bilaplacian_crack_problem::export_solution(const plain_vector &U){
       assert(pts.size() == 1);
       size_type j=pts[0].i;
       scalar_type x3 = P[2];
-      assert(finite(VTHETA[2*j]));
-      //assert(finite(VT[2*j]));
+      assert(std::isfinite(VTHETA[2*j]));
+      //assert(std::isfinite(VT[2*j]));
       V[3*i+0] = - x3 * VTHETA[2*j+0]; //V[3*i+0] = VT[2*j+0] + x3 * VTHETA.at(2*j+0);
       V[3*i+1] = - x3 * VTHETA[2*j+1]; //V[3*i+1] = VT[2*j+1] + x3 * VTHETA[2*j+1];
       V[3*i+2] = V3[j];
-      assert(finite(V[3*i]));
-      assert(finite(V[3*i+1]));
-      assert(finite(V[3*i+2]));
+      assert(std::isfinite(V[3*i]));
+      assert(std::isfinite(V[3*i+1]));
+      assert(std::isfinite(V[3*i+2]));
     }
     
     cout << "export to " << datafilename + "_3d.dx" << "..\n";
