@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2001-2016 Y. Renard, J. Pommier.
+ Copyright (C) 2001-2017 Y. Renard, J. Pommier.
 
  This file is a part of GetFEM++
 
@@ -82,8 +82,9 @@ namespace getfemint {
 
   static std::string dim_of_gfi_array(const gfi_array *t) {
     std::stringstream ss;
-    for (size_type i=0; i < static_cast<size_type>(gfi_array_get_ndim(t)); ++i) {
-      if (i) ss << "x"; ss << gfi_array_get_dim(t)[i];
+    for (size_type i=0; i<static_cast<size_type>(gfi_array_get_ndim(t)); ++i) {
+      if (i) ss << "x";
+      ss << gfi_array_get_dim(t)[i];
     }
     return ss.str();
   }
