@@ -480,7 +480,10 @@ static void do_high_level_generic_assembly(mexargs_in& in, mexargs_out& out) {
   switch (order) {
   case 0:
     workspace.assembly(0);
-    out.pop().from_scalar(workspace.assembled_potential());
+    if (workspace.assembled_tensor().size() == 1)
+      out.pop().from_scalar(workspace.assembled_potential());
+    else
+      out.pop().from_dlvector(workspace.assembled_tensor().as_vector());
     break;
 
   case 1:
@@ -507,7 +510,7 @@ static void do_high_level_generic_assembly(mexargs_in& in, mexargs_out& out) {
   }
 }
 
-static void do_expression_analysis(mexargs_in& in) {
+static void do_expression_analysis(mexargs_in& in, mexargs_out& out) {
 
   std::string expr = in.pop().to_string();
 
@@ -569,6 +572,8 @@ static void do_expression_analysis(mexargs_in& in) {
 
   workspace.add_expression(expr, mim, getfem::dummy_mesh_region(), der_order);
   workspace.print(cout);
+  std::string result = workspace.extract_order0_term();
+  out.pop().from_string(result.c_str());
 }
 
 // To be parallelized
@@ -1204,8 +1209,8 @@ void gf_asm(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
       Analyse a high-level generic assembly expression and print
       information about the provided expression.@*/
     sub_command
-      ("expression analysis", 1, -1, 0, 0,
-       do_expression_analysis(in);
+      ("expression analysis", 1, -1, 0, 1,
+       do_expression_analysis(in, out);
        );
 
 
