@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2012-2015 Tomas Ligursky, Yves Renard, Konstantinos Poulios.
+ Copyright (C) 2012-2017 Tomas Ligursky, Yves Renard, Konstantinos Poulios.
 
  This file is a part of GetFEM++
 
@@ -21,7 +21,8 @@
 
 #include <getfemint_misc.h>
 #include <getfemint_workspace.h>
-#include <getfemint_cont_struct.h>
+#include <getfemint.h>
+#include <getfem/getfem_continuation.h>
 
 using namespace getfemint;
 
@@ -34,7 +35,7 @@ struct sub_gf_cont_struct_get : virtual public dal::static_stored_object {
                    getfem::cont_struct_getfem_model *ps) = 0;
 };
 
-typedef boost::intrusive_ptr<sub_gf_cont_struct_get> psub_command;
+typedef std::shared_ptr<sub_gf_cont_struct_get> psub_command;
 
 // Function to avoid warning in macro with unused arguments.
 template <typename T> static inline void dummy_func(T &) {}
@@ -46,7 +47,7 @@ template <typename T> static inline void dummy_func(T &) {}
                        getfem::cont_struct_getfem_model *ps)                \
       { dummy_func(in); dummy_func(out); dummy_func(ps); code }             \
     };                                                                      \
-    psub_command psubc = new subc;                                          \
+    psub_command psubc = std::make_shared<subc>();			    \
     psubc->arg_in_min = arginmin; psubc->arg_in_max = arginmax;             \
     psubc->arg_out_min = argoutmin; psubc->arg_out_max = argoutmax;         \
     subc_tab[cmd_normalize(name)] = psubc;                                  \
@@ -229,8 +230,8 @@ void gf_cont_struct_get(getfemint::mexargs_in& m_in,
        the non-smooth point. It is supposed that the point given by
        `solution` and `parameter` is a point on a smooth solution branch
        within the distance equal to the minimum step size from the end point
-       of this branch, and the corresponding tangent that is directed towards
-       the end point is given by `tangent_sol` and `tangent_par`.@*/
+       of this branch, and the corresponding tangent given by `tangent_sol`
+       and `tangent_par` is directed towards the end point.@*/
     sub_command
       ("non-smooth branching", 4, 4, 0, 0,
 
@@ -288,7 +289,7 @@ void gf_cont_struct_get(getfemint::mexargs_in& m_in,
 
   if (m_in.narg() < 2)  THROW_BADARG( "Wrong number of input arguments");
 
-  getfem::cont_struct_getfem_model *ps = m_in.pop().to_cont_struct();
+  getfem::cont_struct_getfem_model *ps = to_cont_struct_object(m_in.pop());
   std::string init_cmd   = m_in.pop().to_string();
   std::string cmd        = cmd_normalize(init_cmd);
 
