@@ -476,7 +476,8 @@ void gf_model_get(getfemint::mexargs_in& m_in,
        if (alpha_mult < scalar_type(0))
          alpha_mult = 3.0/5.0;
 
-       getfem::default_newton_line_search default_ls;
+       // getfem::default_newton_line_search default_ls;
+       getfem::newton_search_with_step_control default_ls;
        getfem::simplest_newton_line_search simplest_ls(size_type(-1), alpha_max_ratio,
                                                        alpha_min, alpha_mult, alpha_threshold_res);
        getfem::systematic_newton_line_search systematic_ls(size_type(-1), alpha_min, alpha_mult);
@@ -913,7 +914,7 @@ void gf_model_get(getfemint::mexargs_in& m_in,
       `compute_small_strain_elastoplasticity_Von_Mises`.
       @*/
     sub_command
-      ("small strain elastoplasticity next iter", 10, 15, 0, 0,
+      ("small strain elastoplasticity next iter", 3, 15, 0, 0,
        getfem::mesh_im *mim = to_meshim_object(in.pop());
        std::string lawname = in.pop().to_string();
        filter_lawname(lawname);
@@ -986,7 +987,7 @@ void gf_model_get(getfemint::mexargs_in& m_in,
       before any call of this function.
       @*/
     sub_command 	 
-      ("small strain elastoplasticity Von Mises", 11, 16, 0, 0,
+      ("small strain elastoplasticity Von Mises", 4, 16, 0, 0,
        getfem::mesh_im *mim = to_meshim_object(in.pop());
        const getfem::mesh_fem *mf_vm = to_meshfem_object(in.pop());
        std::string lawname = in.pop().to_string();
