@@ -16,7 +16,7 @@
 % Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
 
 
-function check_asm(iverbose,idebug)
+function check_plasticity(iverbose,idebug)
   global gverbose;
   global gdebug;  
   if (nargin >= 1),
@@ -161,8 +161,8 @@ En = '((Grad_Previous_u+Grad_Previous_u'')/2)';
 expr_sigma = strcat('(', B_inv, '*(Von_Mises_projection((',B,'*((1-alpha)*sigma))+(-(H)*(((1-alpha)*',En,')+(alpha*', Enp1, ')))+(alpha*', ApH, '*(',Enp1,'-',En,')) + (alpha*', ...
     B, '*sigma), von_mises_threshold) + (H)*(((1-alpha)*',En,')+(alpha*', Enp1, '))))');
   
-% gf_model_set(md, 'add nonlinear generic assembly brick', mim, strcat(expr_sigma, 'Grad_u:Grad_Test_u'));
-gf_model_set(md, 'add nonlinear generic assembly brick', mim, strcat(expr_sigma, ':Grad_Test_u'));
+% gf_model_set(md, 'add nonlinear term', mim, strcat(expr_sigma, 'Grad_u:Grad_Test_u'));
+gf_model_set(md, 'add nonlinear term', mim, strcat(expr_sigma, ':Grad_Test_u'));
 % gf_model_set(md, 'add finite strain elasticity brick', mim, 'SaintVenant Kirchhoff', 'u', '[lambda; mu]');
 
 return;
