@@ -26,7 +26,7 @@ mim=gf_mesh_im(m, gf_integ('IM_GAUSS_PARALLELEPIPED(2, 4)'));
 % mim=gf_mesh_im(m, gf_integ('IM_EXACT_PARALLELEPIPED(2)')); % not allowed with the high level generic assembly
 
 border = gf_mesh_get(m,'outer faces');
-gf_mesh_set(m, 'region', 42, border); % create the region (:#(B42
+gf_mesh_set(m, 'region', 42, border); % create the region (:#42
 
 % the boundary edges appears in red
 gf_plot_mesh(m, 'regions', [42], 'vertices','on','convexes','on'); 
@@ -34,7 +34,7 @@ gf_plot_mesh(m, 'regions', [42], 'vertices','on','convexes','on');
 
 md=gf_model('real');
 gf_model_set(md, 'add fem variable', 'u', mf);
-% gf_model_set(md, 'add linear generic assembly brick', mim, 'Grad_u.Grad_Test_u');
+% gf_model_set(md, 'add linear term', mim, 'Grad_u.Grad_Test_u');
 gf_model_set(md, 'add Laplacian brick', mim, 'u');
 R=gf_mesh_fem_get(mf, 'eval', {'(x-.5).^2 + (y-.5).^2 + x/5 - y/3'});
 gf_model_set(md, 'add initialized fem data', 'DirichletData', mf, R);
