@@ -251,15 +251,15 @@ namespace bgeot {
 
   static size_type simplexified_prism_6_nb = 6;
  
-  static size_type simplexified_pyramid[30] = {
+  static size_type simplexified_pyramid[8] = {
      0, 1, 2, 4, 3, 2, 1, 4
   };
 
-  static size_type simplexified_pyramid_nb = 3;
+  static size_type simplexified_pyramid_nb = 2;
    
   
-  size_type simplexified_tab(pconvex_structure cvs,
-                             size_type **tab) {
+  size_type simplexified_tab(pconvex_structure cvs, size_type **tab) {
+
     if (cvs == parallelepiped_structure(2)) {
       *tab = simplexified_parallelepiped_2;
       return simplexified_parallelepiped_2_nb;
@@ -285,32 +285,33 @@ namespace bgeot {
       return simplexified_parallelepiped_6_nb;
     }
 
-    if (cvs == prism_structure(3)) {
+    if (cvs == prism_P1_structure(3)) {
       *tab = simplexified_prism_3;
       return simplexified_prism_3_nb;
     }
 
-    if (cvs == prism_structure(4)) {
+    if (cvs == prism_P1_structure(4)) {
       *tab = simplexified_prism_4;
       return simplexified_prism_4_nb;
     }
 
-    if (cvs == prism_structure(5)) {
+    if (cvs == prism_P1_structure(5)) {
       *tab = simplexified_prism_5;
       return simplexified_prism_5_nb;
     }
 
-    if (cvs == prism_structure(6)) {
+    if (cvs == prism_P1_structure(6)) {
       *tab = simplexified_prism_6;
       return simplexified_prism_6_nb;
     }
 
-    if (cvs == pyramidal_structure(1)) {
+    if (cvs == pyramid_QK_structure(1)) {
       *tab = simplexified_pyramid;
       return simplexified_pyramid_nb;
     }
 
-    GMM_ASSERT1(false, "No simplexification for this element");
+    *tab = nullptr; // Not taken into account
+    return 0;
   }
 
 
