@@ -299,7 +299,7 @@ namespace bgeot {
             : convex_product_structure(cv1->faces_structure()[i], cv2);
 
         faces[i] = std::vector<short_type>(cv1->nb_points_of_face(i)
-                                              * cv2->nb_points());
+                                           * cv2->nb_points());
 
         for (short_type j = 0; j < cv1->nb_points_of_face(i); j++)
           for (short_type l = 0; l < cv2->nb_points(); l++) {
@@ -361,18 +361,23 @@ namespace bgeot {
   DAL_DOUBLE_KEY(parallelepiped_key_, dim_type, dim_type);
 
   pconvex_structure parallelepiped_structure(dim_type nc, dim_type k) {
-    if (nc <= 1) return simplex_structure(nc, k);
+    if (nc <= 1)
+      return simplex_structure(nc, k);
+
     dal::pstatic_stored_object_key
       pcsk = std::make_shared<parallelepiped_key_>(nc, k);
 
     dal::pstatic_stored_object o = dal::search_stored_object(pcsk);
-    if (o) return ((std::dynamic_pointer_cast<const parallelepiped_>(o))->p);
-    auto p = std::make_shared<parallelepiped_>();
-    p->p = convex_product_structure(parallelepiped_structure(dim_type(nc-1),k),
-                                    simplex_structure(1,k));
-    dal::add_stored_object(pcsk, dal::pstatic_stored_object(p), p->p,
-                           dal::PERMANENT_STATIC_OBJECT);
-    return p->p;
+    if (o)
+      return ((std::dynamic_pointer_cast<const parallelepiped_>(o))->p);
+    else {
+      auto p = std::make_shared<parallelepiped_>();
+      p->p = convex_product_structure(parallelepiped_structure(dim_type(nc-1),k),
+                                      simplex_structure(1,k));
+      dal::add_stored_object(pcsk, dal::pstatic_stored_object(p), p->p,
+                             dal::PERMANENT_STATIC_OBJECT);
+      return p->p;
+    }
   }
 
   /* ******************************************************************** */
@@ -398,7 +403,7 @@ namespace bgeot {
     p->Nc = nc;
     p->nbpt = (nc == 2) ? 8 : 20;
     p->nbf =  (nc == 2) ? 4 : 6;
-    p->basic_pcvs =  parallelepiped_structure(nc);
+    p->basic_pcvs = parallelepiped_structure(nc); // k=1
     p->faces_struct.resize(p->nbf);
     p->faces = std::vector< std::vector<short_type> >(p->nbf);
     p->dir_points_ = std::vector<short_type>(p->Nc + 1);
@@ -409,10 +414,10 @@ namespace bgeot {
       // 3     4
       // |     |
       // 0--1--2
-      sc(p->faces[0]) = 2,4,7;
-      sc(p->faces[1]) = 0,3,5;
-      sc(p->faces[2]) = 5,6,7;
-      sc(p->faces[3]) = 0,1,2;
+      p->faces[0] = {2,4,7};
+      p->faces[1] = {0,3,5};
+      p->faces[2] = {5,6,7};
+      p->faces[3] = {0,1,2};
 
       p->dir_points_[0] = 0;
       p->dir_points_[1] = 2;
@@ -429,14 +434,14 @@ namespace bgeot {
       // 8 /       9 /
       // |/        |/
       // 0----1----2
-      sc(p->faces[0]) = 2,4,7,9,11,14,16,19;
-      sc(p->faces[1]) = 0,3,5,8,10,12,15,17;
+      p->faces[0] = {2,4,7,9,11,14,16,19};
+      p->faces[1] = {0,3,5,8,10,12,15,17};
 
-      sc(p->faces[2]) = 5,6,7,10,11,17,18,19;
-      sc(p->faces[3]) = 0,1,2,8,9,12,13,14;
+      p->faces[2] = {5,6,7,10,11,17,18,19};
+      p->faces[3] = {0,1,2,8,9,12,13,14};
 
-      sc(p->faces[4]) = 12,13,14,15,16,17,18,19;
-      sc(p->faces[5]) = 0,1,2,3,4,5,6,7;
+      p->faces[4] = {12,13,14,15,16,17,18,19};
+      p->faces[5] = {0,1,2,3,4,5,6,7};
 
       p->dir_points_[0] = 0;
       p->dir_points_[1] = 2;
@@ -460,32 +465,32 @@ namespace bgeot {
   /*        Pyramidal 3D structure for k=1 or 2.                          */
   /* ******************************************************************** */
 
-  struct pyramidal_structure_ : public convex_structure {
-    friend pconvex_structure pyramidal_structure(dim_type k);
+  struct pyramid_QK_structure_ : public convex_structure {
+    friend pconvex_structure pyramid_QK_structure(dim_type k);
   };
 
-  DAL_SIMPLE_KEY(pyramidal_structure_key_, dim_type);
+  DAL_SIMPLE_KEY(pyramid_QK_structure_key_, dim_type);
 
-  pconvex_structure pyramidal_structure(dim_type k) {
+  pconvex_structure pyramid_QK_structure(dim_type k) {
     GMM_ASSERT1(k == 1 || k == 2, "Sorry, pyramidal elements implemented "
-		"only for degree one or two.");
+                "only for degree one or two.");
     dal::pstatic_stored_object_key
-      pcsk = std::make_shared<pyramidal_structure_key_>(k);
+      pcsk = std::make_shared<pyramid_QK_structure_key_>(k);
     dal::pstatic_stored_object o = dal::search_stored_object(pcsk);
-    if (o) return std::dynamic_pointer_cast<const convex_structure>(o);
+    if (o)
+      return std::dynamic_pointer_cast<const convex_structure>(o);
 
-    auto p = std::make_shared<pyramidal_structure_>();
+    auto p = std::make_shared<pyramid_QK_structure_>();
     pconvex_structure pcvs(p);
-    
+
     p->Nc = 3;
     p->dir_points_ = std::vector<short_type>(p->Nc + 1);
-    
 
     if (k == 1) {
       p->nbpt = 5;
       p->nbf = 5;
       p->auto_basic = true;
-      //    4 
+      //    4
       //   /|||
       //  / || |
       // 2-|--|-3
@@ -495,11 +500,11 @@ namespace bgeot {
       // 0------1
       p->faces_struct.resize(p->nbf);
       p->faces = std::vector< std::vector<short_type> >(p->nbf);
-      sc(p->faces[0]) = 0,1,2,3;
-      sc(p->faces[1]) = 0,1,4;
-      sc(p->faces[2]) = 1,3,4;
-      sc(p->faces[3]) = 3,2,4;
-      sc(p->faces[4]) = 2,0,4;
+      p->faces[0] = {0,1,2,3};
+      p->faces[1] = {0,1,4};
+      p->faces[2] = {1,3,4};
+      p->faces[3] = {3,2,4};
+      p->faces[4] = {2,0,4};
 
       p->dir_points_[0] = 0;
       p->dir_points_[1] = 1;
@@ -508,16 +513,16 @@ namespace bgeot {
 
       p->faces_struct[0] = parallelepiped_structure(2);
       for (int i = 1; i < p->nbf; i++)
-	p->faces_struct[i] = simplex_structure(2);
+        p->faces_struct[i] = simplex_structure(2);
 
       dal::add_stored_object(pcsk, pcvs, parallelepiped_structure(2),
-			     simplex_structure(2),
-			     dal::PERMANENT_STATIC_OBJECT);
-      
+                             simplex_structure(2),
+                             dal::PERMANENT_STATIC_OBJECT);
+
     } else {
       p->nbpt = 14;
       p->nbf = 5;
-      p->basic_pcvs = pyramidal_structure(1);
+      p->basic_pcvs = pyramid_QK_structure(1);
       //    13
       //   /  |
       //  11--12
@@ -531,11 +536,11 @@ namespace bgeot {
       // 0--1--2
       p->faces_struct.resize(p->nbf);
       p->faces = std::vector< std::vector<short_type> >(p->nbf);
-      sc(p->faces[0]) = 0,1,2,3,4,5,6,7,8;
-      sc(p->faces[1]) = 0,1,2,9,10,13;
-      sc(p->faces[2]) = 2,5,8,10,12,13;
-      sc(p->faces[3]) = 8,7,6,12,11,13;
-      sc(p->faces[4]) = 6,3,0,11,9,13;
+      p->faces[0] = {0,1,2,3,4,5,6,7,8};
+      p->faces[1] = {0,1,2,9,10,13};
+      p->faces[2] = {2,5,8,10,12,13};
+      p->faces[3] = {8,7,6,12,11,13};
+      p->faces[4] = {6,3,0,11,9,13};
 
       p->dir_points_[0] = 0;
       p->dir_points_[1] = 2;
@@ -544,15 +549,136 @@ namespace bgeot {
 
       p->faces_struct[0] = parallelepiped_structure(2, 2);
       for (int i = 1; i < p->nbf; i++)
-	p->faces_struct[i] = simplex_structure(2, 2);
+        p->faces_struct[i] = simplex_structure(2, 2);
 
       dal::add_stored_object(pcsk, pcvs, parallelepiped_structure(2, 2),
-			     simplex_structure(2, 2),
-			     dal::PERMANENT_STATIC_OBJECT);
+                             simplex_structure(2, 2),
+                             dal::PERMANENT_STATIC_OBJECT);
     }
     return pcvs;
   }
-  
+
+  /* ******************************************************************** */
+  /*        Incomplete quadratic pyramidal 3D structure.                  */
+  /* ******************************************************************** */
+
+  struct pyramid_Q2_incomplete_structure_ : public convex_structure {
+    friend pconvex_structure pyramid_Q2_incomplete_structure();
+  };
+
+  DAL_SIMPLE_KEY(pyramid_Q2_incomplete_structure_key_, dim_type);
+
+  pconvex_structure pyramid_Q2_incomplete_structure() {
+    dal::pstatic_stored_object_key
+      pcsk = std::make_shared<pyramid_Q2_incomplete_structure_key_>(0);
+    dal::pstatic_stored_object o = dal::search_stored_object(pcsk);
+    if (o)
+      return std::dynamic_pointer_cast<const convex_structure>(o);
+
+    auto p = std::make_shared<pyramid_Q2_incomplete_structure_>();
+    pconvex_structure pcvs(p);
+
+    p->Nc = 3;
+    p->dir_points_ = std::vector<short_type>(p->Nc + 1);
+
+    p->nbpt = 13;
+    p->nbf = 5;
+    p->basic_pcvs = pyramid_QK_structure(1);
+    //    12
+    //   /  |
+    //  10--11
+    //  |   |
+    //  8---9
+    //  /    |
+    // 5--6--7
+    // |     |
+    // 3     4
+    // |     |
+    // 0--1--2
+    p->faces_struct.resize(p->nbf);
+    p->faces = std::vector< std::vector<short_type> >(p->nbf);
+    p->faces[0] = {0,1,2,3,4,5,6,7};
+    p->faces[1] = {0,1,2,8,9,12};
+    p->faces[2] = {2,4,7,9,11,12};
+    p->faces[3] = {7,6,5,11,10,12};
+    p->faces[4] = {5,3,0,10,8,12};
+
+    p->dir_points_[0] = 0;
+    p->dir_points_[1] = 2;
+    p->dir_points_[2] = 5;
+    p->dir_points_[3] = 12;
+
+    p->faces_struct[0] = Q2_incomplete_structure(2);
+    for (int i = 1; i < p->nbf; i++)
+      p->faces_struct[i] = simplex_structure(2, 2);
+
+    dal::add_stored_object(pcsk, pcvs, Q2_incomplete_structure(2),
+                           simplex_structure(2, 2),
+                           dal::PERMANENT_STATIC_OBJECT);
+    return pcvs;
+  }
+
+  /* ******************************************************************** */
+  /*        Incomplete quadratic triangular prism 3D structure.           */
+  /* ******************************************************************** */
+
+  struct prism_incomplete_P2_structure_ : public convex_structure {
+    friend pconvex_structure prism_incomplete_P2_structure();
+  };
+
+  DAL_SIMPLE_KEY(prism_incomplete_P2_structure_key_, dim_type);
+
+  pconvex_structure prism_incomplete_P2_structure() {
+    dal::pstatic_stored_object_key
+      pcsk = std::make_shared<prism_incomplete_P2_structure_key_>(0);
+    dal::pstatic_stored_object o = dal::search_stored_object(pcsk);
+    if (o)
+      return std::dynamic_pointer_cast<const convex_structure>(o);
+
+    auto p = std::make_shared<prism_incomplete_P2_structure_>();
+    pconvex_structure pcvs(p);
+
+    p->Nc = 3;
+    p->dir_points_ = std::vector<short_type>(p->Nc + 1);
+
+    p->nbpt = 15;
+    p->nbf = 5;
+    p->basic_pcvs = prism_P1_structure(3);
+    //    14
+    //    /|`
+    //  12 | 13
+    //  /  8  `
+    // 9--10--11
+    // |   |   |
+    // |   5   |
+    // 6  / `  7
+    // | 3   4 |
+    // |/     `|
+    // 0---1---2
+    p->faces_struct.resize(p->nbf);
+    p->faces = std::vector< std::vector<short_type> >(p->nbf);
+    p->faces[0] = {2,4,5,7,8,11,13,14};
+    p->faces[1] = {0,3,5,6,8,9,12,14};
+    p->faces[2] = {0,1,2,6,7,9,10,11};
+    p->faces[3] = {9,10,11,12,13,14};
+    p->faces[4] = {0,1,2,3,4,5};
+
+    p->dir_points_[0] = 0;
+    p->dir_points_[1] = 2;
+    p->dir_points_[2] = 5;
+    p->dir_points_[3] = 9;
+
+    for (int i = 0; i < 3; i++)
+      p->faces_struct[i] = Q2_incomplete_structure(2);
+    p->faces_struct[3] = simplex_structure(2, 2);
+    p->faces_struct[4] = simplex_structure(2, 2);
+
+    dal::add_stored_object(pcsk, pcvs, simplex_structure(2, 2),
+                           Q2_incomplete_structure(2),
+                           dal::PERMANENT_STATIC_OBJECT);
+    return pcvs;
+  }
+
   /* ******************************************************************** */
   /*        Generic dummy convex with n global nodes.                     */
   /* ******************************************************************** */
