@@ -51,7 +51,7 @@ namespace bgeot
   bool read_until(std::istream &ist, const char *st);
 
   /** Very simple lexical analysis of general interest for reading small
-   *  langages with a "MATLAB like" syntax :
+   *  languages with a "MATLAB like" syntax :
    *  spaces are ignored, '%' indicates a commentary until the end of the line,
    *  '...' indicates that the instruction continue on the next line
    *  (or separate two sub part of the same character string).
@@ -69,8 +69,8 @@ namespace bgeot
    *  allows to consider the carriage return as a space character.
    */
   int get_token(std::istream &ist, std::string &st,
-		bool ignore_cr = true, bool to_up = true, 
-		bool read_un_pm = true, int *linenb = 0);
+                bool ignore_cr = true, bool to_up = true,
+                bool read_un_pm = true, int *linenb = 0);
 
   struct skip {
     const char *s;
@@ -84,11 +84,11 @@ namespace bgeot
   int casecmp(const char *a, const char *b, unsigned n=unsigned(-1));
 
   inline int casecmp(const std::string& a, const char *b,
-		     unsigned n=unsigned(-1))
+                     unsigned n=unsigned(-1))
   { return casecmp(a.c_str(),b,n); }
 
   inline int casecmp(const std::string& a, const std::string& b,
-		     unsigned n=unsigned(-1))
+                     unsigned n=unsigned(-1))
   { return casecmp(a.c_str(), b.c_str(),n); }
 
   inline int casecmp(char a, char b)
@@ -97,14 +97,14 @@ namespace bgeot
   /* ********************************************************************* */
   /*       Read a parameter file.                                          */
   /* ********************************************************************* */
- 
-  // The associated langage has approximatively the following grammar:
+
+  // The associated language has approximatively the following grammar:
   //
   // 'instruction' := 'parameter_name' '=' 'expression';
   //              or 'if' 'expression'
   //                 'instruction_list'
   //                 [ 'else' 'instruction_list' ]
-  //                 'end'          
+  //                 'end'
   // 'expression' := '[' 'expression' ',' ... ']'
   //              or 'parameter_name'
   //              or 'numeric_value'
@@ -136,23 +136,23 @@ namespace bgeot
       double real_value;
       std::string string_value;
       std::vector<param_value> array_value;
-      
+
     public :
-      param_type type_of_param(void) const { return pt; }
-      double &real(void) { return real_value; }
-      double real(void) const { return real_value; }
-      std::string &string(void) { return string_value; }
-      const std::string &string(void) const { return string_value; }
-      std::vector<param_value> &array(void) { return array_value; }
-      const std::vector<param_value> &array(void) const { return array_value; }
+      param_type type_of_param() const { return pt; }
+      double &real() { return real_value; }
+      double real() const { return real_value; }
+      std::string &string() { return string_value; }
+      const std::string &string() const { return string_value; }
+      std::vector<param_value> &array() { return array_value; }
+      const std::vector<param_value> &array() const { return array_value; }
       param_value(double e = 0.0) : pt(REAL_VALUE), real_value(e) {}
       param_value(std::string s) : pt(STRING_VALUE), real_value(0.0),
-				   string_value(s) {}
+                                   string_value(s) {}
       param_value(char *s) :  pt(STRING_VALUE), real_value(0.0),
-			      string_value(s) {}
+                              string_value(s) {}
       param_value(param_type p): pt(p), real_value(0.0) {}
     };
-    
+
   protected :
 
     std::map<std::string, param_value> parameters;
@@ -161,7 +161,7 @@ namespace bgeot
     std::string current_file;
 
     int get_next_token(std::istream &f);
-    void valid_token(void);
+    void valid_token();
     std::string temp_string;
     param_value read_expression_list(std::istream &f, bool skipped);
     param_value read_expression(std::istream &f, bool skipped);
@@ -170,31 +170,33 @@ namespace bgeot
     void parse_error(const std::string &t);
     void syntax_error(const std::string &t);
     void do_bin_op(std::vector<md_param::param_value> &value_list,
-		   std::vector<int> &op_list, std::vector<int> &prior_list);
-    
-  public :
-      
+                   std::vector<int> &op_list, std::vector<int> &prior_list);
 
-    double real_value(const std::string &name, const char *comment = 0);
-    long int_value(const std::string &name, const char *comment = 0);
+  public :
+
+    double real_value(const std::string &name, const char *comment = 0,
+                      double default_val=0.);
+    long int_value(const std::string &name, const char *comment = 0,
+                   long default_val=0);
     const std::string &string_value(const std::string &name,
-				    const char *comment = 0);
+                                    const char *comment = 0,
+                                    const std::string &default_val="");
     const std::vector<param_value> &array_value(const std::string &name,
-						const char *comment = 0);
+                                                const char *comment = 0);
     void add_int_param(const std::string &name, long e)
     { parameters[name] = param_value(double(e)); }
     void add_real_param(const std::string &name, double e)
     { parameters[name] = param_value(e); }
     void add_string_param(const std::string &name, const std::string &s)
     { parameters[name] = param_value(s); }
- 
+
     // todo : add the access to the arrays
-    
+
     void read_param_file(std::istream &f);
 
-    /** Read the parameters on the command line. If a name is found the 
-     *	corresponding .param file is searched. If a -dNOM=VALUE is found
-     *	(or -d NOM=VALUE), it is evaluated.
+    /** Read the parameters on the command line. If a name is found the
+     *  corresponding .param file is searched. If a -dNOM=VALUE is found
+     *  (or -d NOM=VALUE), it is evaluated.
      */
     void read_command_line(int argc, char *argv[]);
   };
@@ -203,7 +205,7 @@ namespace bgeot
 
 
 namespace ftool {
- 
+
   // For compatibility with Getfem 2.0
 
   using bgeot::md_param;
