@@ -115,18 +115,18 @@ namespace bgeot {
   public :
 
     /// Dimension of the reference element.
-    dim_type dim(void) const { return cvr->structure()->dim(); }
+    dim_type dim() const { return cvr->structure()->dim(); }
     /// True if the transformation is linear (affine in fact).
-    bool is_linear(void) const { return is_lin; }
+    bool is_linear() const { return is_lin; }
     /// Number of geometric nodes.
-    size_type nb_points(void) const { return cvr->nb_points(); }
+    size_type nb_points() const { return cvr->nb_points(); }
     /// Pointer on the convex of reference.
-    pconvex_ref convex_ref(void) const { return cvr; }
+    pconvex_ref convex_ref() const { return cvr; }
     /// Structure of the reference element.
-    pconvex_structure structure(void) const { return cvr->structure(); }
+    pconvex_structure structure() const { return cvr->structure(); }
     /// Basic structure of the reference element.
-    pconvex_structure basic_structure(void) const
-    { return bgeot::basic_structure(cvr->structure()); }
+    pconvex_structure basic_structure() const
+      { return bgeot::basic_structure(cvr->structure()); }
     /// Gives the value of the functions vector at a certain point.
     virtual void poly_vector_val(const base_node &pt, base_vector &val) const = 0;
     /// Gives the value of a subgroup of the functions vector at a certain point.
@@ -142,17 +142,17 @@ namespace bgeot {
     /// compute K matrix from multiplication of G with gradient
     virtual void compute_K_matrix(const base_matrix &G, const base_matrix &pc, base_matrix &K) const;
     /// Gives the number of vertices.
-    size_type nb_vertices(void) const { return vertices_.size(); }
+    size_type nb_vertices() const { return vertices_.size(); }
     /// Gives the indices of vertices between the nodes.
-    const std::vector<size_type> &vertices(void) const { return vertices_; }
+    const std::vector<size_type> &vertices() const { return vertices_; }
     /// Gives the array of geometric nodes (on reference convex)
-    const stored_point_tab &geometric_nodes(void) const
+    const stored_point_tab &geometric_nodes() const
     { return cvr->points(); }
     /// Gives the array of geometric nodes (on reference convex)
-    pstored_point_tab pgeometric_nodes(void) const
+    pstored_point_tab pgeometric_nodes() const
     { return cvr->pspt(); }
     /// Gives the array of the normals to faces (on reference convex)
-    const std::vector<base_small_vector> &normals(void) const
+    const std::vector<base_small_vector> &normals() const
     { return cvr->normals(); }
     /** Apply the geometric transformation to point pt,
         PTAB contains the points of the real convex */
@@ -161,9 +161,9 @@ namespace bgeot {
     base_node transform(const base_node &pt, const base_matrix &G) const;
     /** Compute the gradient at point x, pc is resized to [nb_points() x dim()]
         if the transformation is linear, x is not used at all */
-    size_type complexity(void) const { return complexity_; }
+    size_type complexity() const { return complexity_; }
     virtual ~geometric_trans()
-    { DAL_STORED_OBJECT_DEBUG_DESTROYED(this, "Geometric transformation"); }
+      { DAL_STORED_OBJECT_DEBUG_DESTROYED(this, "Geometric transformation"); }
     geometric_trans()
       { DAL_STORED_OBJECT_DEBUG_CREATED(this, "Geometric transformation"); }
   };
@@ -221,7 +221,11 @@ namespace bgeot {
   pgeometric_trans APIDECL linear_product_geotrans(pgeometric_trans pg1,
                                            pgeometric_trans pg2);
   pgeometric_trans APIDECL Q2_incomplete_geotrans(dim_type nc);
-  pgeometric_trans APIDECL pyramidal_geotrans(short_type k);
+  pgeometric_trans APIDECL prism_incomplete_P2_geotrans();
+  pgeometric_trans APIDECL pyramid_QK_geotrans(short_type k);
+  IS_DEPRECATED inline pgeometric_trans APIDECL
+  pyramid_geotrans(short_type k) { return pyramid_QK_geotrans(k); }
+  pgeometric_trans APIDECL pyramid_Q2_incomplete_geotrans();
 
   /**
      Get the geometric transformation from its string name.
@@ -233,12 +237,17 @@ namespace bgeot {
 
      List of possible names:
      GT_PK(N,K)   : Transformation on simplexes, dim N, degree K
-     
-     GT_QK(N,K)   : Transformation on parallelepipeds, dim N, degree K
-     GT_PRISM(N,K)          : Transformation on prisms, dim N, degree K
-     GT_Q2_INCOMPLETE(N)    : Q2 incomplete transformation in dim N=2 or 3.
-     GT_PRODUCT(a,b)        : tensorial product of two transformations
-     GT_LINEAR_PRODUCT(a,b) : Linear tensorial product of two transformations
+
+     GT_QK(N,K)         : Transformation on parallelepipeds, dim N, degree K
+     GT_PRISM(N,K)      : Transformation on prisms, dim N, degree K
+     GT_PYRAMID_QK(K)   : Transformation on pyramids, dim 3, degree K=0,1,2
+     GT_Q2_INCOMPLETE(N)      : Q2 incomplete transformation in dim N=2 or 3.
+     GT_PYRAMID_Q2_INCOMPLETE : incomplete quadratic pyramid transformation
+                                in dim 3
+     GT_PRISM_INCOMPLETE_P2   : incomplete quadratic prism transformation in
+                                dim 3
+     GT_PRODUCT(a,b)          : tensorial product of two transformations
+     GT_LINEAR_PRODUCT(a,b)   : Linear tensorial product of two transformations
      GT_LINEAR_QK(N) : shortcut for GT_LINEAR_PRODUCT(GT_LINEAR_QK(N-1),
                                                       GT_PK(1,1))
    */
@@ -251,14 +260,14 @@ namespace bgeot {
    *
    *  pt is the position of the evaluation point on the reference element
    */
-  base_small_vector APIDECL compute_normal(const geotrans_interpolation_context& c,
-                                   size_type face);
+  base_small_vector APIDECL
+  compute_normal(const geotrans_interpolation_context& c, size_type face);
 
   /** return the local basis (i.e. the normal in the first column, and the
    *  tangent vectors in the other columns
    */
-  base_matrix
-  APIDECL compute_local_basis(const geotrans_interpolation_context& c,
+  base_matrix APIDECL
+  compute_local_basis(const geotrans_interpolation_context& c,
                       size_type face);
     //@}
 
@@ -399,6 +408,7 @@ namespace bgeot {
     mutable base_node xref_;  /** reference point */
     mutable base_node xreal_; /** transformed point */
     const base_matrix *G_;    /** pointer to the matrix of real nodes of the convex */
+    mutable base_node cv_center_; /** real center of convex (average of columns of G) */
     mutable base_matrix K_, B_, B3_, B32_; /** see documentation (getfem kernel doc) for more details */
     pgeometric_trans pgt_;
     pgeotrans_precomp pgp_;
@@ -407,9 +417,9 @@ namespace bgeot {
     mutable scalar_type J_, J__; /** Jacobian */
     mutable base_matrix PC, B_factors;
     mutable base_vector aux1, aux2;
-    mutable std::vector<int> ipvt;
-    mutable bool have_J_, have_B_, have_B3_, have_B32_, have_K_;
-    void compute_J(void) const;
+    mutable std::vector<long> ipvt;
+    mutable bool have_J_, have_B_, have_B3_, have_B32_, have_K_, have_cv_center_;
+    void compute_J() const;
   public:
     bool have_xref() const { return !xref_.empty(); }
     bool have_xreal() const { return !xreal_.empty(); }
@@ -424,6 +434,8 @@ namespace bgeot {
     const base_node& xref() const;
     /// coordinates of the current point, in the real convex.
     const base_node& xreal() const;
+    /// coordinates of the center of the real convex.
+    const base_node& cv_center() const;
     /// See getfem kernel doc for these matrices
     const base_matrix& K() const;
     const base_matrix& B() const;
@@ -434,75 +446,81 @@ namespace bgeot {
     const base_matrix& G() const { return *G_; }
     /** get the Jacobian of the geometric trans (taken at point @c xref() ) */
     scalar_type J() const { if (!have_J_) compute_J(); return J_; }
-    size_type N() const { if (have_G()) return G().nrows();
+    size_type N() const {
+      if (have_G()) return G().nrows();
       else if (have_xreal()) return xreal_.size();
-      else GMM_ASSERT2(false, "cannot get N"); return 0; }
+      else GMM_ASSERT2(false, "cannot get N");
+      return 0;
+    }
     size_type ii() const { return ii_; }
     bgeot::pgeotrans_precomp pgp() const { return pgp_; }
     /** change the current point (assuming a geotrans_precomp_ is used) */
     void set_ii(size_type ii__) {
       if (ii_ != ii__) {
-	if (pgt_ && !pgt()->is_linear())
-	  { have_K_ = have_B_ = have_B3_ = have_B32_ = have_J_ = false; }
-	xref_.resize(0); xreal_.resize(0);
-	ii_=ii__;
+        if (pgt_ && !pgt()->is_linear())
+          { have_K_ = have_B_ = have_B3_ = have_B32_ = have_J_ = false; }
+        xref_.resize(0); xreal_.resize(0);
+        ii_=ii__;
       }
     }
     /** change the current point (coordinates given in the reference convex) */
     void set_xref(const base_node& P);
     void change(bgeot::pgeotrans_precomp pgp__,
-		size_type ii__,
-		const base_matrix& G__) {
+                size_type ii__,
+                const base_matrix& G__) {
       G_ = &G__; pgt_ = pgp__->get_trans(); pgp_ = pgp__;
       pspt_ = pgp__->get_ppoint_tab(); ii_ = ii__;
       have_J_ = have_B_ = have_B3_ = have_B32_ = have_K_ = false;
-      xref_.resize(0); xreal_.resize(0);
+      have_cv_center_ = false;
+      xref_.resize(0); xreal_.resize(0); cv_center_.resize(0);
     }
     void change(bgeot::pgeometric_trans pgt__,
-		bgeot::pstored_point_tab pspt__,
-		size_type ii__,
-		const base_matrix& G__) {
+                bgeot::pstored_point_tab pspt__,
+                size_type ii__,
+                const base_matrix& G__) {
       G_ = &G__; pgt_ = pgt__; pgp_ = 0; pspt_ = pspt__; ii_ = ii__;
       have_J_ = have_B_ = have_B3_ = have_B32_ = have_K_ = false;
-      xref_.resize(0); xreal_.resize(0);
+      have_cv_center_ = false;
+      xref_.resize(0); xreal_.resize(0); cv_center_.resize(0);
     }
     void change(bgeot::pgeometric_trans pgt__,
-		const base_node& xref__,
-		const base_matrix& G__) {
+                const base_node& xref__,
+                const base_matrix& G__) {
       xref_ = xref__; G_ = &G__; pgt_ = pgt__; pgp_ = 0; pspt_ = 0;
       ii_ = size_type(-1);
       have_J_ = have_B_ = have_B3_ = have_B32_ = have_K_ = false;
-      xreal_.resize(0);
+      have_cv_center_ = false;
+      xreal_.resize(0); cv_center_.resize(0);
     }
-    
+
     geotrans_interpolation_context()
       : G_(0), pgt_(0), pgp_(0), pspt_(0), ii_(size_type(-1)),
       have_J_(false), have_B_(false), have_B3_(false), have_B32_(false),
-      have_K_(false) {}
+      have_K_(false), have_cv_center_(false) {}
     geotrans_interpolation_context(bgeot::pgeotrans_precomp pgp__,
                                    size_type ii__,
                                    const base_matrix& G__)
       : G_(&G__), pgt_(pgp__->get_trans()), pgp_(pgp__),
       pspt_(pgp__->get_ppoint_tab()), ii_(ii__), have_J_(false), have_B_(false),
-      have_B3_(false), have_B32_(false), have_K_(false) {}
+      have_B3_(false), have_B32_(false), have_K_(false), have_cv_center_(false) {}
     geotrans_interpolation_context(bgeot::pgeometric_trans pgt__,
                                    bgeot::pstored_point_tab pspt__,
                                    size_type ii__,
                                    const base_matrix& G__)
       : G_(&G__), pgt_(pgt__), pgp_(0),
       pspt_(pspt__), ii_(ii__), have_J_(false), have_B_(false), have_B3_(false),
-      have_B32_(false), have_K_(false) {}
+      have_B32_(false), have_K_(false), have_cv_center_(false) {}
     geotrans_interpolation_context(bgeot::pgeometric_trans pgt__,
                                    const base_node& xref__,
                                    const base_matrix& G__)
       : xref_(xref__), G_(&G__), pgt_(pgt__), pgp_(0), pspt_(0),
       ii_(size_type(-1)),have_J_(false), have_B_(false), have_B3_(false),
-      have_B32_(false), have_K_(false)  {}
+      have_B32_(false), have_K_(false), have_cv_center_(false) {}
   };
 
   /* Function allowing the add of an geometric transformation method outwards
      of getfem_integration.cc */
-  
+
   typedef dal::naming_system<geometric_trans>::param_list gt_param_list;
 
   void APIDECL add_geometric_trans_name
@@ -519,12 +537,12 @@ namespace bgeot {
   // Optimized matrix mult for small matrices.
   // Multiply the matrix A of size MxN by B of size NxP in C of size MxP
   void mat_mult(const scalar_type *A, const scalar_type *B, scalar_type *C,
-		size_type M, size_type N, size_type P);
+                size_type M, size_type N, size_type P);
   // Optimized matrix mult for small matrices.
   // Multiply the matrix A of size MxN by the transpose of B of size PxN
   // in C of size MxP
   void mat_tmult(const scalar_type *A, const scalar_type *B, scalar_type *C,
-		 size_type M, size_type N, size_type P);
+                 size_type M, size_type N, size_type P);
 
 }  /* end of namespace bgeot.                                             */
 
