@@ -72,12 +72,18 @@ namespace getfem {
     const stored_mesh_slice *psl;
     std::unique_ptr<mesh_fem> pmf;
     dal::bit_vector pmf_dof_used;
-    std::vector<unsigned> pmf_cell_type;
+    std::vector<unsigned> pmf_mapping_type;
     std::ofstream real_os;
     dim_type dim_;
     bool reverse_endian;
     enum { EMPTY, HEADER_WRITTEN, STRUCTURE_WRITTEN, IN_CELL_DATA,
            IN_POINT_DATA } state;
+
+    template<class T> void write_val(T v);
+    template<class V> void write_vec(V p, size_type qdim);
+    template<class IT> void write_3x3tensor(IT p);
+    void write_separ();
+
   public:
     typedef enum { VTK_VERTEX = 1,
                    VTK_LINE = 3,
@@ -94,10 +100,11 @@ namespace getfem {
                    VTK_QUADRATIC_QUAD = 23,
                    VTK_QUADRATIC_TETRA = 24,
                    VTK_QUADRATIC_HEXAHEDRON = 25,
-                   /*VTK_QUADRATIC_WEDGE = 26,*/
-		   VTK_QUADRATIC_PYRAMID = 27,
+                   VTK_QUADRATIC_WEDGE = 26,
+                   VTK_QUADRATIC_PYRAMID = 27,
                    VTK_BIQUADRATIC_QUAD = 28,
-                   VTK_TRIQUADRATIC_HEXAHEDRON = 29 } vtk_cell_type;
+                   VTK_TRIQUADRATIC_HEXAHEDRON = 29,
+                   VTK_BIQUADRATIC_QUADRATIC_WEDGE = 32 } vtk_cell_type;
     vtk_export(const std::string& fname, bool ascii_ = false);
     vtk_export(std::ostream &os_, bool ascii_ = false);
 
@@ -136,7 +143,7 @@ namespace getfem {
         convex_index().card() elements.  */
 
     template<class VECT> void write_cell_data(const VECT& U,
-                                              const std::string& name, 
+                                              const std::string& name,
                                               size_type qdim = 1);
     /** export a data_set correspounding to measures of quality for each convex
         of the supplied mesh (which should have the same number of convex than
@@ -157,10 +164,6 @@ namespace getfem {
     void write_mesh_structure_from_mesh_fem();
     void switch_to_cell_data();
     void switch_to_point_data();
-    template<class T> void write_val(T v);
-    template<class V> void write_vec(V p, size_type qdim);
-    template<class IT> void write_3x3tensor(IT p);
-    void write_separ();
     template<class VECT> void write_dataset_(const VECT& U,
                                              const std::string& name,
                                              size_type qdim,
