@@ -1,7 +1,7 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
 
- Copyright (C) 1999-2015 Yves Renard
+ Copyright (C) 1999-2017 Yves Renard
 
  This file is a part of GetFEM++
 
@@ -50,13 +50,21 @@
    element QK on a parallelepiped.
 
    - "FEM_Q2_INCOMPLETE(N)" : incomplete Q2 elements with 8 and 20 dof
-                              (serendipity Quad 8 and Hexa 20 elements)
+   (serendipity Quad 8 and Hexa 20 elements)
 
-   - "FEM_PK_PRISM(N,K)" : classical Lagrange element PK on a prism.
+   - "FEM_Q2_INCOMPLETE_DISCONTINUOUS(N)" : discontinuous incomplete Q2
+   elements with 8 and 20 dof (serendipity Quad 8 and Hexa 20 elements)
 
-   - "FEM_PK_PRISM_DISCONTINUOUS(N,K,alpha)" : classical discontinuous
+   - "FEM_PRISM_PK(N,K)" : classical Lagrange element PK on a prism.
+
+   - "FEM_PRISM_PK_DISCONTINUOUS(N,K,alpha)" : classical discontinuous
    Lagrange element PK on a prism.
 
+   - "FEM_PRISM_INCOMPLETE_P2" : Incomplete Lagrange element on a
+   quadratic 3D prism (serendipity, 15-node wedge element). Can be connected
+   toa standard P2 Lagrange on its triangular faces and a Q2_INCOMPLETE
+   Lagrange element on its quadrangular faces.
+
    - "FEM_PK_WITH_CUBIC_BUBBLE(N,K)" : classical Lagrange element PK
    on a simplex with an additional volumic bubble function.
 
@@ -88,22 +96,35 @@
    - "FEM_REDUCED_QUADC1_COMPOSITE" : quadrilateral element, composite
       P3 element and C^1 (12 dof).
 
-   - "FEM_PK_HIERARCHICAL(N,K)" : PK element with a hierarchical basis
+   - "FEM_PK_HIERARCHICAL(N,K)" : PK element with a hierarchical basis.
 
-   - "FEM_QK_HIERARCHICAL(N,K)" : QK element with a hierarchical basis
+   - "FEM_QK_HIERARCHICAL(N,K)" : QK element with a hierarchical basis.
 
-   - "FEM_PK_PRISM_HIERARCHICAL(N,K)" : PK element on a prism with a
-   hierarchical basis
+   - "FEM_PRISM_PK_HIERARCHICAL(N,K)" : PK element on a prism with a
+   hierarchical basis.
 
    - "FEM_STRUCTURED_COMPOSITE(FEM, K)" : Composite fem on a grid with
-   K divisions
+   K divisions.
 
    - "FEM_PK_HIERARCHICAL_COMPOSITE(N,K,S)" : PK composite element on
-   a grid with S subdivisions and with a hierarchical basis
+   a grid with S subdivisions and with a hierarchical basis.
 
    - "FEM_PK_FULL_HIERARCHICAL_COMPOSITE(N,K,S)" : PK composite
    element with S subdivisions and a hierarchical basis on both degree
-   and subdivision
+   and subdivision.
+
+   - "FEM_PYRAMID_QK(K)" : Lagrange element on a 3D pyramid of degree
+   K=0, 1 or 2. Can be connected to a standard P1/P2 Lagrange element on its
+   triangular faces and a standard Q1/Q2 Lagrange element on its quadrangular
+   face.
+
+   - "FEM_PYRAMID_QK_DISCONTINUOUS(K)" : Discontinuous Lagrange element
+   on a 3D pyramid of degree K = 0, 1 or 2.
+
+   - "FEM_PYRAMID_Q2_INCOMPLETE" : Incomplete Lagrange element on a
+   quadratic 3D pyramid (serendipity, 13-node element). Can be connected to
+   a standard P2 Lagrange element on its triangular faces and a Q2_INCOMPLETE
+   Lagrange element on its quadrangular face.
 
 */
 
@@ -186,7 +207,7 @@ namespace getfem {
   /// Returns the xfem_index of dof (0 for normal dof)
   size_type dof_xfem_index(pdof_description);
 
-  size_type reserve_xfem_index(void);
+  size_type reserve_xfem_index();
   dim_type coord_index_of_dof(pdof_description a);
 
   int dof_weak_compatibility(pdof_description a, pdof_description b);
@@ -218,15 +239,16 @@ namespace getfem {
   class virtual_fem;
 
   /** type of pointer on a fem description @see getfem::virtual_fem */
-  typedef boost::intrusive_ptr<const getfem::virtual_fem> pfem;
+  typedef std::shared_ptr<const getfem::virtual_fem> pfem;
 
   class fem_precomp_;
-  typedef boost::intrusive_ptr<const getfem::fem_precomp_> pfem_precomp;
+  typedef std::shared_ptr<const getfem::fem_precomp_> pfem_precomp;
 
   class fem_interpolation_context;
 
   /** @brief Base class for finite element description */
-  class virtual_fem : virtual public dal::static_stored_object {
+  class virtual_fem : virtual public dal::static_stored_object,
+                      public std::enable_shared_from_this<const virtual_fem> {
   public :
     enum vec_type { VECTORIAL_NOTRANSFORM_TYPE, VECTORIAL_PRIMAL_TYPE,
                     VECTORIAL_DUAL_TYPE };
@@ -239,7 +261,8 @@ namespace getfem {
        But cvs_node has to be a pointer, as bgeot::convex_structure
        inherits from dal::static_stored_object
     */
-    boost::intrusive_ptr<bgeot::convex_structure> cvs_node;
+    std::shared_ptr<bgeot::convex_structure> cvs_node;
+    std::vector<std::vector<short_type>> face_tab; // face list for each dof
     bgeot::convex<base_node> cv_node;
     mutable bgeot::pstored_point_tab pspt;
     mutable bool pspt_valid;
@@ -251,6 +274,7 @@ namespace getfem {
     bool is_pol;            // true if the FEM is polynomial
     bool is_polycomp;       // true if the FEM is polynomial composite
     bool real_element_defined;
+    bool is_standard_fem;   // is_equiv && !real_element_defined && scalar
     short_type es_degree;   // estimated polynomial degree of the FEM
     short_type hier_raff;   // hierarchical refinement of the FEM
     vec_type vtype; // for vectorial elements, type of transformation
@@ -261,7 +285,7 @@ namespace getfem {
     /** Number of degrees of freedom.
 
        @param cv the convex number for this FEM. This information is
-       rarely used, by is needed by some "special" FEMs, such as
+       rarely used, but is needed by some "special" FEMs, such as
        getfem::interpolated_fem.
     */
     virtual size_type nb_dof(size_type /*cv*/) const
@@ -275,16 +299,16 @@ namespace getfem {
     size_type nb_components(size_type cv) const
       { return nb_dof(cv) * ntarget_dim; }
     /// Get the array of pointer on dof description.
-    const std::vector<pdof_description> &dof_types(void) const
+    const std::vector<pdof_description> &dof_types() const
       { return dof_types_; }
-    short_type hierarchical_raff(void) const { return hier_raff; }
+    short_type hierarchical_raff() const { return hier_raff; }
     /// dimension of the reference element.
-    dim_type dim(void) const { return dim_; }
-    dim_type &dim(void) { return dim_; }
+    dim_type dim() const { return dim_; }
+    dim_type &dim() { return dim_; }
     /// dimension of the target space.
-    dim_type target_dim(void) const { return ntarget_dim; }
+    dim_type target_dim() const { return ntarget_dim; }
     /// Type of vectorial element.
-    vec_type vectorial_type(void) const { return vtype; }
+    vec_type vectorial_type() const { return vtype; }
     /// Return the convex of the reference element.
     virtual bgeot::pconvex_ref ref_convex(size_type) const { return cvr; }
     /// @internal
@@ -298,8 +322,8 @@ namespace getfem {
     /// Gives the convex structure of the reference element nodes.
     bgeot::pconvex_structure structure(size_type cv) const
     { return node_convex(cv).structure(); }
-    const std::string &debug_name(void) const { return debug_name_; }
-    std::string &debug_name(void) { return debug_name_; }
+    const std::string &debug_name() const { return debug_name_; }
+    std::string &debug_name() { return debug_name_; }
     virtual bgeot::pstored_point_tab node_tab(size_type) const {
       if (!pspt_valid) {
         pspt = bgeot::store_point_tab(cv_node.points());
@@ -315,21 +339,25 @@ namespace getfem {
     */
     const base_node &node_of_dof(size_type cv, size_type i) const
       { return (*(node_tab(cv)))[i];}
-    bool is_on_real_element(void) const { return real_element_defined; }
-    bool is_equivalent(void) const { return is_equiv; }
-    bool need_G(void) const
+    virtual const std::vector<short_type> &
+    faces_of_dof(size_type /*cv*/, size_type i) const;
+    bool is_on_real_element() const { return real_element_defined; }
+    bool is_equivalent() const { return is_equiv; }
+    bool need_G() const
     { return !(is_equivalent()) || real_element_defined; }
     /// true if the base functions are such that @f$ \varphi_i(\textrm{node\_of\_dof(j)}) = \delta_{ij} @f$
-    bool is_lagrange(void) const { return is_lag; }
+    bool is_lagrange() const { return is_lag; }
     /// true if the base functions are polynomials
-    bool is_polynomial(void) const { return is_pol; }
-    bool is_polynomialcomp(void) const { return is_polycomp; }
-    bool &is_polynomialcomp(void) { return is_polycomp; }
-    bool &is_equivalent(void) { return is_equiv; }
-    bool &is_lagrange(void) { return is_lag; }
-    bool &is_polynomial(void) { return is_pol; }
-    short_type estimated_degree(void) const { return es_degree; }
-    short_type &estimated_degree(void) { return es_degree; }
+    bool is_polynomial() const { return is_pol; }
+    bool is_polynomialcomp() const { return is_polycomp; }
+    bool is_standard() const { return is_standard_fem; }
+    bool &is_polynomialcomp() { return is_polycomp; }
+    bool &is_equivalent() { return is_equiv; }
+    bool &is_lagrange() { return is_lag; }
+    bool &is_polynomial() { return is_pol; }
+    bool &is_standard() { return is_standard_fem; }
+    short_type estimated_degree() const { return es_degree; }
+    short_type &estimated_degree() { return es_degree; }
 
     virtual void mat_trans(base_matrix &, const base_matrix &,
                            bgeot::pgeometric_trans) const
@@ -439,47 +467,29 @@ namespace getfem {
     void add_node(const pdof_description &d, const base_node &pt,
                   const dal::bit_vector &faces);
     void add_node(const pdof_description &d, const base_node &pt);
-    void init_cvs_node(void);
-    void unfreeze_cvs_node(void);
+    void init_cvs_node();
+    void unfreeze_cvs_node();
 
     virtual_fem &operator =(const virtual_fem &f) {
       copy(f); return *this;
     }
 
-    virtual_fem(void) {
+    virtual_fem() {
+      DAL_STORED_OBJECT_DEBUG_CREATED(this, "Fem");
       ntarget_dim = 1; dim_ = 1;
-      is_equiv = is_pol = is_polycomp = is_lag = false;
+      is_equiv = is_pol = is_polycomp = is_lag = is_standard_fem = false;
       pspt_valid = false; hier_raff = 0; real_element_defined = false;
       es_degree = 5;
       vtype = VECTORIAL_NOTRANSFORM_TYPE;
       cvs_node = bgeot::new_convex_structure();
     }
-    virtual_fem(const virtual_fem& f) : dal::static_stored_object()
-    { copy(f); }
-    virtual ~virtual_fem() {}
+    virtual_fem(const virtual_fem& f)
+      : dal::static_stored_object(),
+        std::enable_shared_from_this<const virtual_fem>()
+    { copy(f); DAL_STORED_OBJECT_DEBUG_CREATED(this, "Fem"); }
+    virtual ~virtual_fem() { DAL_STORED_OBJECT_DEBUG_DESTROYED(this, "Fem"); }
   private:
-    void copy(const virtual_fem &f) {
-      dof_types_ = f.dof_types_;
-
-      cvs_node = bgeot::new_convex_structure();
-      *cvs_node = *f.cvs_node; // deep copy
-      cv_node = f.cv_node;
-      cv_node.structure() = cvs_node;
-      pspt = 0;
-      pspt_valid = false;
-      cvr = f.cvr;
-      dim_ = f.dim_;
-      ntarget_dim = f.ntarget_dim;
-      vtype = f.vtype;
-      is_equiv = f.is_equiv;
-      is_lag = f.is_lag;
-      is_pol = f.is_pol;
-      is_polycomp = f.is_polycomp;
-      real_element_defined = f.real_element_defined;
-      es_degree = f.es_degree;
-      hier_raff = f.hier_raff;
-      debug_name_ = f.debug_name_;
-    }
+    void copy(const virtual_fem &f);
   };
 
   /**
@@ -490,12 +500,48 @@ namespace getfem {
   template <class FUNC> class fem : public virtual_fem {
   protected :
     std::vector<FUNC> base_;
+    mutable std::vector<std::vector<FUNC>> grad_, hess_;
+    mutable bool grad_computed_;
+    mutable bool hess_computed_;
 
+    void compute_grad_() const {
+      auto guard = getfem::omp_guard{};
+      if (grad_computed_) return;
+      size_type R = nb_base_components(0);
+      dim_type n = dim();
+      grad_.resize(R);
+      for (size_type i = 0; i < R; ++i) {
+	grad_[i].resize(n);
+	for (dim_type j = 0; j < n; ++j) {
+	  grad_[i][j] = base_[i]; grad_[i][j].derivative(j);
+	}
+      }
+      grad_computed_ = true;
+    }
+
+    void compute_hess_() const {
+      auto guard = getfem::omp_guard{};
+      if (hess_computed_) return;
+      size_type R = nb_base_components(0);
+      dim_type n = dim();
+      hess_.resize(R);
+      for (size_type i = 0; i < R; ++i) {
+	hess_[i].resize(n*n);
+	for (dim_type j = 0; j < n; ++j) {
+	  for (dim_type k = 0; k < n; ++k) {
+	    hess_[i][j+k*n] = base_[i];
+	    hess_[i][j+k*n].derivative(j); hess_[i][j+k*n].derivative(k);
+	  }
+	}
+      }
+      hess_computed_ = true;
+    }
+    
   public :
 
     /// Gives the array of basic functions (components).
-    const std::vector<FUNC> &base(void) const { return base_; }
-    std::vector<FUNC> &base(void) { return base_; }
+    const std::vector<FUNC> &base() const { return base_; }
+    std::vector<FUNC> &base() { return base_; }
     /** Evaluates at point x, all base functions and returns the result in
         t(nb_base,target_dim) */
     void base_value(const base_node &x, base_tensor &t) const {
@@ -511,6 +557,7 @@ namespace getfem {
         reference element directions 0,..,dim-1 and returns the result in
         t(nb_base,target_dim,dim) */
     void grad_base_value(const base_node &x, base_tensor &t) const {
+      if (!grad_computed_) compute_grad_();
       bgeot::multi_index mi(3);
       dim_type n = dim();
       mi[2] = n; mi[1] = target_dim(); mi[0] = short_type(nb_base(0));
@@ -519,12 +566,13 @@ namespace getfem {
       base_tensor::iterator it = t.begin();
       for (dim_type j = 0; j < n; ++j)
         for (size_type i = 0; i < R; ++i, ++it)
-          { FUNC f = base_[i]; f.derivative(j); *it = bgeot::to_scalar(f.eval(x.begin())); }
+	  *it = bgeot::to_scalar(grad_[i][j].eval(x.begin()));
     }
     /** Evaluates at point x, the hessian of all base functions w.r.t. the
         reference element directions 0,..,dim-1 and returns the result in
         t(nb_base,target_dim,dim,dim) */
     void hess_base_value(const base_node &x, base_tensor &t) const {
+      if (!hess_computed_) compute_hess_();
       bgeot::multi_index mi(4);
       dim_type n = dim();
       mi[3] = n; mi[2] = n; mi[1] = target_dim();
@@ -534,28 +582,33 @@ namespace getfem {
       base_tensor::iterator it = t.begin();
       for (dim_type k = 0; k < n; ++k)
         for (dim_type j = 0; j < n; ++j)
-          for (size_type i = 0; i < R; ++i, ++it) {
-            FUNC f = base_[i];
-            f.derivative(j); f.derivative(k);
-            *it = bgeot::to_scalar(f.eval(x.begin()));
-          }
+          for (size_type i = 0; i < R; ++i, ++it)
+	    *it = bgeot::to_scalar(hess_[i][j+k*n].eval(x.begin()));
     }
 
+    fem() : grad_computed_(false), hess_computed_(false){}
+
   };
 
   /** Classical polynomial FEM. */
-  typedef const fem<base_poly> * ppolyfem;
+  typedef const fem<bgeot::base_poly> * ppolyfem;
   /** Polynomial composite FEM */
   typedef const fem<bgeot::polynomial_composite> * ppolycompfem;
+  /** Rational fration FEM */
+  typedef const fem<bgeot::base_rational_fraction> * prationalfracfem;
 
   /** Give a pointer on the structures describing the classical
       polynomial fem of degree k on a given convex type.
 
       @param pgt the geometric transformation (which defines the convex type).
       @param k the degree of the fem.
+      @param complete a flag which requests complete Langrange polynomial
+      elements even if the provided pgt is an incomplete one (e.g. 8-node
+      quadrilateral or 20-node hexahedral).
       @return a ppolyfem.
   */
-  pfem classical_fem(bgeot::pgeometric_trans pgt, short_type k);
+  pfem classical_fem(bgeot::pgeometric_trans pgt, short_type k,
+                     bool complete=false);
 
   /** Give a pointer on the structures describing the classical
       polynomial discontinuous fem of degree k on a given convex type.
@@ -568,12 +621,17 @@ namespace getfem {
       0, the nodes are located as usual (i.e. with node on the convex border),
       and for 0 < alpha < 1, they converge to the center of gravity of the convex.
 
+      @param complete a flag which requests complete Langrange polynomial
+      elements even if the provided pgt is an incomplete one (e.g. 8-node
+      quadrilateral or 20-node hexahedral).
+
       @return a ppolyfem.
   */
-  pfem classical_discontinuous_fem(bgeot::pgeometric_trans pg, short_type k, scalar_type alpha=0);
+  pfem classical_discontinuous_fem(bgeot::pgeometric_trans pg, short_type k,
+                                   scalar_type alpha=0, bool complete=false);
 
   /** get a fem descriptor from its string name. */
-  pfem fem_descriptor(std::string name);
+  pfem fem_descriptor(const std::string &name);
 
   /** get the string name of a fem descriptor. */
   std::string name_of_fem(pfem p);
@@ -589,11 +647,11 @@ namespace getfem {
   */
   class fem_precomp_ : virtual public dal::static_stored_object {
   protected:
-    pfem pf;
-    bgeot::pstored_point_tab pspt;
-    mutable std::vector<base_tensor> c;   // store values of base functions
-    mutable std::vector<base_tensor> pc;  // store gradients of base functions
-    mutable std::vector<base_tensor> hpc; // store hessians of base functions
+    const pfem pf;
+    const bgeot::pstored_point_tab pspt;
+    mutable std::vector<base_tensor> c;   // stored values of base functions
+    mutable std::vector<base_tensor> pc;  // stored gradients of base functions
+    mutable std::vector<base_tensor> hpc; // stored hessians of base functions
   public:
     /// returns values of the base functions
     inline const base_tensor &val(size_type i) const
@@ -605,9 +663,12 @@ namespace getfem {
     inline const base_tensor &hess(size_type i) const
       { if (hpc.empty()) init_hess(); return hpc[i]; }
     inline pfem get_pfem() const { return pf; }
-    inline const bgeot::stored_point_tab& get_point_tab() const
-      { return *pspt; }
-    fem_precomp_(pfem, bgeot::pstored_point_tab);
+    // inline const bgeot::stored_point_tab& get_point_tab() const
+    //  { return *pspt; }
+    inline bgeot::pstored_point_tab get_ppoint_tab() const
+    { return pspt; }
+    fem_precomp_(const pfem, const bgeot::pstored_point_tab);
+    ~fem_precomp_() { DAL_STORED_OBJECT_DEBUG_DESTROYED(this, "Fem_precomp"); }
   private:
     void init_val() const;
     void init_grad() const;
@@ -668,7 +729,7 @@ namespace getfem {
       precomps.insert(p);
       return p;
     }
-    void clear(void);
+    void clear();
     ~fem_precomp_pool() { clear(); }
   };
 
@@ -685,6 +746,7 @@ namespace getfem {
     pfem_precomp pfp_;      // optional fem_precomp_ (speed up the computations)
     size_type convex_num_;  // The element (convex) number
     short_type face_num_;   // Face number for boundary integration
+    int xfem_side_; // For the computation of a jump with fem_level_set only
   public:
     /// true if a fem_precomp_ has been supplied.
     bool have_pfp() const { return pfp_ != 0; }
@@ -696,10 +758,14 @@ namespace getfem {
         at point @c this->xref())
     */
     void base_value(base_tensor& t, bool withM = true) const;
+    // Optimized function for high level generic assembly
+    void pfp_base_value(base_tensor& t, const pfem_precomp &pfp__);
     /** fill the tensor with the gradient of the base functions (taken
         at point @c this->xref())
     */
     void grad_base_value(base_tensor& t, bool withM = true) const;
+    // Optimized function for high level generic assembly
+    void pfp_grad_base_value(base_tensor& t, const pfem_precomp &pfp__);
     /** fill the tensor with the hessian of the base functions (taken
         at point @c this->xref())
     */
@@ -709,6 +775,9 @@ namespace getfem {
     /** get the current convex number */
     size_type convex_num() const;
     bool is_convex_num_valid() const;
+    void invalid_convex_num() { convex_num_ = size_type(-1); }
+    /** set the current face number */
+    void set_face_num(short_type f) { face_num_ = f; }
     /** get the current face number */
     short_type face_num() const;
     /** On a face ? */
@@ -717,23 +786,61 @@ namespace getfem {
     pfem_precomp pfp() const { return pfp_; }
     void set_pfp(pfem_precomp newpfp);
     void set_pf(pfem newpf);
-    fem_interpolation_context();
+    int xfem_side() const { return xfem_side_; }
+    void set_xfem_side(int side) { xfem_side_ = side; }
+    void change(bgeot::pgeotrans_precomp pgp__,
+		pfem_precomp pfp__, size_type ii__,
+		const base_matrix& G__, size_type convex_num__,
+		short_type face_num__ = short_type(-1)) {
+      bgeot::geotrans_interpolation_context::change(pgp__,ii__,G__);
+      convex_num_ = convex_num__; face_num_ = face_num__;  xfem_side_ = 0;
+      set_pfp(pfp__);
+    }
+    void change(bgeot::pgeometric_trans pgt__,
+		pfem_precomp pfp__, size_type ii__,
+		const base_matrix& G__, size_type convex_num__,
+		short_type face_num__ = short_type(-1)) {
+      bgeot::geotrans_interpolation_context::change
+	(pgt__, pfp__->get_ppoint_tab(), ii__, G__);
+      convex_num_ = convex_num__; face_num_ = face_num__; xfem_side_ = 0;
+      set_pfp(pfp__);
+    }
+    void change(bgeot::pgeometric_trans pgt__,
+		pfem pf__, const base_node& xref__, const base_matrix& G__,
+		size_type convex_num__,	short_type face_num__=short_type(-1)) {
+      bgeot::geotrans_interpolation_context::change(pgt__,xref__,G__);
+      pf_ = pf__; pfp_ = 0; convex_num_ = convex_num__; face_num_ = face_num__;
+      xfem_side_ = 0;
+    }
+    fem_interpolation_context()
+      : bgeot::geotrans_interpolation_context(),
+	convex_num_(size_type(-1)), face_num_(short_type(-1)), xfem_side_(0) {}
     fem_interpolation_context(bgeot::pgeotrans_precomp pgp__,
                               pfem_precomp pfp__, size_type ii__,
                               const base_matrix& G__,
                               size_type convex_num__,
-                              short_type face_num__ = short_type(-1));
+                              short_type face_num__ = short_type(-1))
+      : bgeot::geotrans_interpolation_context(pgp__,ii__,G__),
+	convex_num_(convex_num__), face_num_(face_num__), xfem_side_(0)
+    { set_pfp(pfp__); }
     fem_interpolation_context(bgeot::pgeometric_trans pgt__,
                               pfem_precomp pfp__, size_type ii__,
                               const base_matrix& G__,
                               size_type convex_num__,
-                              short_type face_num__ = short_type(-1));
+                              short_type face_num__ = short_type(-1))
+      : bgeot::geotrans_interpolation_context(pgt__,pfp__->get_ppoint_tab(),
+					      ii__, G__),
+	convex_num_(convex_num__), face_num_(face_num__), xfem_side_(0)
+    { set_pfp(pfp__); }
     fem_interpolation_context(bgeot::pgeometric_trans pgt__,
                               pfem pf__,
                               const base_node& xref__,
                               const base_matrix& G__,
                               size_type convex_num__,
-                              short_type face_num__ = short_type(-1));
+                              short_type face_num__ = short_type(-1))
+      : bgeot::geotrans_interpolation_context(pgt__,xref__,G__),
+	pf_(pf__), pfp_(0), convex_num_(convex_num__), face_num_(face_num__),
+	xfem_side_(0) {}
   };
 
   // IN : coeff(Qmult,nb_dof)
