@@ -1,7 +1,7 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
 
- Copyright (C) 2013-2016 Yves Renard
+ Copyright (C) 2013-2017 Yves Renard
 
  This file is a part of GetFEM++
 
@@ -32,7 +32,7 @@
 /** @file getfem_generic_assembly.h
     @author  Yves Renard <Yves.Renard@insa-lyon.fr>
     @date November 18, 2013.
-    @brief A langage for generic assembly of pde boundary value problems.
+    @brief A language for generic assembly of pde boundary value problems.
  */
 
 
@@ -40,8 +40,8 @@
 #define GETFEM_GENERIC_ASSEMBLY_H__
 
 #include <map>
-#include "getfem/getfem_models.h"
 #include "getfem/getfem_interpolation.h"
+#include "getfem/getfem_mesh_slice.h"
 
 
 #ifdef _WIN32
@@ -54,14 +54,136 @@
 
 namespace getfem {
 
-  class ga_tree;
+  struct ga_tree;
+  class model;
+  class ga_workspace;
 
+  typedef gmm::rsvector<scalar_type> model_real_sparse_vector;
+  typedef gmm::rsvector<complex_type> model_complex_sparse_vector;
+  typedef std::vector<scalar_type> model_real_plain_vector;
+  typedef std::vector<complex_type> model_complex_plain_vector;
+
+  typedef gmm::col_matrix<model_real_sparse_vector> model_real_sparse_matrix;
+  typedef gmm::col_matrix<model_complex_sparse_vector>
+  model_complex_sparse_matrix;
+
+  typedef gmm::row_matrix<model_real_sparse_vector>
+  model_real_row_sparse_matrix;
+  typedef gmm::row_matrix<model_complex_sparse_vector>
+  model_complex_row_sparse_matrix;
+  
+  // 0 : ok
+  // 1 : function or operator name or "X"
+  // 2 : reserved prefix Grad, Hess, Div, Test and Test2
   int ga_check_name_validity(const std::string &name);
 
+  //=========================================================================
+  //  Virtual interpolate_transformation object.
+  //=========================================================================
+
+  struct var_trans_pair {
+    std::string varname, transname;
+    bool operator <(const var_trans_pair &vt) const {
+      return (varname < vt.varname) ||
+             (!(varname > vt.varname) && transname < vt.transname);
+    }
+    var_trans_pair() : varname(), transname() {}
+    var_trans_pair(const std::string &v, const std::string &t)
+      : varname(v), transname(t) {}
+  };
+
+  class APIDECL virtual_interpolate_transformation {
+
+  public:
+    virtual void extract_variables
+    (const ga_workspace &workspace, std::set<var_trans_pair> &vars,
+     bool ignore_data, const mesh &m,
+     const std::string &interpolate_name) const = 0;
+    virtual void init(const ga_workspace &workspace) const = 0;
+    virtual int transform
+    (const ga_workspace &workspace, const mesh &m,
+     fem_interpolation_context &ctx_x, const base_small_vector &Normal,
+     const mesh **m_t, size_type &cv, short_type &face_num,
+     base_node &P_ref, base_small_vector &N_y,
+     std::map<var_trans_pair, base_tensor> &derivatives,
+     bool compute_derivatives) const = 0;
+    virtual void finalize() const = 0;
+    virtual std::string expression(void) const { return std::string(); }
+
+    virtual ~virtual_interpolate_transformation() {}
+  };
+
+  typedef std::shared_ptr<const virtual_interpolate_transformation>
+  pinterpolate_transformation;
+
+  //=========================================================================
+  //  Virtual elementary_transformation object.
+  //=========================================================================
+
+  class APIDECL virtual_elementary_transformation {
+
+  public:
+    
+    virtual void give_transformation(const mesh_fem &mf, size_type cv,
+                                     base_matrix &M) const = 0;
+    virtual ~virtual_elementary_transformation() {}
+  };
+
+  typedef std::shared_ptr<const virtual_elementary_transformation>
+  pelementary_transformation;
+
+  //=========================================================================
+  // Structure dealing with macros.
+  //=========================================================================
+
+  class ga_macro {
+
+  protected:
+    ga_tree *ptree;
+    std::string macro_name_;
+    size_type nbp;
+
+  public:
+    ga_macro();
+    ga_macro(const std::string &name, const ga_tree &t, size_type nbp_);
+    ga_macro(const ga_macro &);
+    ~ga_macro();
+    ga_macro &operator =(const ga_macro &);
+
+    const std::string &name() const { return macro_name_; }
+    std::string &name() { return macro_name_; }
+    size_type nb_params() const { return nbp; }
+    size_type &nb_params() { return nbp; }
+    const ga_tree& tree() const { return *ptree; }
+    ga_tree& tree() { return *ptree; }
+  };
+
+
+  class ga_macro_dictionnary {
+
+  protected:
+    const ga_macro_dictionnary *parent;
+    std::map<std::string, ga_macro> macros;
+
+  public:
+    bool macro_exists(const std::string &name) const;
+    const ga_macro &get_macro(const std::string &name) const;
+    
+    void add_macro(const ga_macro &gam);
+    void add_macro(const std::string &name, const std::string &expr);
+    void del_macro(const std::string &name);
+    
+    ga_macro_dictionnary() : parent(0) {}
+    ga_macro_dictionnary(bool, const ga_macro_dictionnary& gamd)
+      : parent(&gamd) {}
+    
+  };
+
   //=========================================================================
   // Structure dealing with predefined operators.
   //=========================================================================
 
+
   struct ga_nonlinear_operator {
 
     typedef std::vector<const base_tensor *> arg_list;
@@ -87,6 +209,7 @@ namespace getfem {
     void add_method(const std::string &name,
                     const std::shared_ptr<ga_nonlinear_operator> &pt)
     { tab[name] = pt; }
+    ga_predef_operator_tab();
   };
 
   //=========================================================================
@@ -156,17 +279,23 @@ namespace getfem {
 
     struct tree_description { // CAUTION: Specific copy constructor
       size_type order; // 0: potential, 1: weak form, 2: tangent operator
+      // -1 : interpolation/ assignment all order,
+      // -2 : assignment on potential, -3 : assignment on weak form
+      // -3 : assignment on tangent operator
+      size_type interpolation; // O : assembly, 1 : interpolate before assembly
+                               // 2 : interpolate after assembly. 
+      std::string varname_interpolation; // Where to interpolate
       std::string name_test1, name_test2;
       std::string interpolate_name_test1, interpolate_name_test2;
       const mesh_im *mim;
       const mesh *m;
       const mesh_region *rg;
       ga_tree *ptree;
-      base_vector elem;
       tree_description()
-        : name_test1(""), name_test2(""),
+        : interpolation(0), varname_interpolation(""),
+          name_test1(""), name_test2(""),
           interpolate_name_test1(""), interpolate_name_test2(""),
-          mim(0), m(0), rg(0), ptree(0), elem(0) {}
+          mim(0), m(0), rg(0), ptree(0) {}
       void copy(const tree_description& td);
       tree_description(const tree_description& td) { copy(td); }
       tree_description &operator =(const tree_description& td);
@@ -194,7 +323,8 @@ namespace getfem {
     std::vector<tree_description> trees;
 
     std::map<std::string, std::vector<std::string> > variable_groups;
-    std::map<std::string, std::string> macros;
+
+    ga_macro_dictionnary macro_dict;
 
     struct m_tree {
       ga_tree *ptree;
@@ -206,27 +336,28 @@ namespace getfem {
       ~m_tree();
     };
 
-    mutable std::map<std::string, m_tree> macro_trees;
-
     void add_tree(ga_tree &tree, const mesh &m, const mesh_im &mim,
                   const mesh_region &rg,
-                  const std::string &expr, size_type add_derivative_order = 2,
-                  bool scalar_expr = true);
+                  const std::string &expr, size_type add_derivative_order,
+                  bool scalar_expr, size_type for_interpolation,
+		  const std::string varname_interpolation);
 
 
     std::shared_ptr<model_real_sparse_matrix> K;
     model_real_sparse_matrix unreduced_K;
     std::shared_ptr<base_vector> V;
     base_vector unreduced_V;
-    scalar_type E = scalar_type(0.);
     base_tensor assemb_t;
+    bool include_empty_int_pts = false;
 
   public:
 
     const model_real_sparse_matrix &assembled_matrix() const { return *K;}
     model_real_sparse_matrix &assembled_matrix() { return *K; }
-    scalar_type &assembled_potential() { return E; }
-    const scalar_type &assembled_potential() const { return E; }
+    scalar_type &assembled_potential()
+    { GMM_ASSERT1(assemb_t.size() == 1, "Bad result size"); return assemb_t[0]; }
+    const scalar_type &assembled_potential() const
+    { GMM_ASSERT1(assemb_t.size() == 1, "Bad result size"); return assemb_t[0]; }
     const base_vector &assembled_vector() const { return *V; }
     base_vector &assembled_vector() { return *V; }
     void set_assembled_matrix(model_real_sparse_matrix &K_) {
@@ -254,10 +385,16 @@ namespace getfem {
     /* Internal use */
     void add_function_expression(const std::string &expr);
     /* Internal use */
-    void add_interpolation_expression(const std::string &expr, const mesh &m,
-                                      const mesh_region &rg=mesh_region::all_convexes());
-    void add_interpolation_expression(const std::string &expr, const mesh_im &mim,
-                                      const mesh_region &rg=mesh_region::all_convexes());
+    void add_interpolation_expression
+    (const std::string &expr, const mesh &m,
+     const mesh_region &rg = mesh_region::all_convexes());
+    void add_interpolation_expression
+    (const std::string &expr, const mesh_im &mim,
+     const mesh_region &rg = mesh_region::all_convexes());
+    void add_assignment_expression
+    (const std::string &dataname, const std::string &expr,
+     const mesh_region &rg_ = mesh_region::all_convexes(),
+     size_type order = 1, bool before = false);
 
     /** Delete all previously added expressions. */
     void clear_expressions();
@@ -282,8 +419,10 @@ namespace getfem {
     void add_im_data(const std::string &name, const im_data &imd,
                      const model_real_plain_vector &VV);
 
-    bool used_variables(model::varnamelist &vl, model::varnamelist &vl_test1,
-                        model::varnamelist &vl_test2, model::varnamelist &dl,
+    bool used_variables(std::vector<std::string> &vl,
+			std::vector<std::string> &vl_test1,
+                        std::vector<std::string> &vl_test2,
+			std::vector<std::string> &dl,
                         size_type order);
 
     bool variable_exists(const std::string &name) const;
@@ -325,19 +464,20 @@ namespace getfem {
     bgeot::multi_index qdims(const std::string &name) const;
 
     const model_real_plain_vector &value(const std::string &name) const;
-
     scalar_type get_time_step() const;
 
     // macros
-    bool macro_exists(const std::string &name) const;
+    bool macro_exists(const std::string &name) const
+    { return macro_dict.macro_exists(name); }
 
     void add_macro(const std::string &name, const std::string &expr)
-    { macros[name] = expr; }
+    { macro_dict.add_macro(name, expr); }
+
+    void del_macro(const std::string &name) { macro_dict.del_macro(name); }
 
     const std::string& get_macro(const std::string &name) const;
 
-    ga_tree& macro_tree(const std::string &name, size_type meshdim,
-                        size_type ref_elt_dim, bool ignore_X) const;
+    const ga_macro_dictionnary &macro_dictionnary() const { return macro_dict; }
 
 
     // interpolate and elementary transformations
@@ -368,18 +508,13 @@ namespace getfem {
 
     void assembly(size_type order);
 
+    void set_include_empty_int_points(bool include);
+    bool include_empty_int_points() const;
 
-    ga_workspace(const getfem::model &md_, bool enable_all_variables = false)
-      : md(&md_), parent_workspace(0),
-        enable_all_md_variables(enable_all_variables)
-    { init(); }
-    ga_workspace(bool, const ga_workspace &gaw)
-      : md(0), parent_workspace(&gaw), enable_all_md_variables(false)
-    { init(); }
-    ga_workspace()
-      : md(0), parent_workspace(0), enable_all_md_variables(false)
-    { init(); }
-    ~ga_workspace() { clear_expressions(); }
+    ga_workspace(const getfem::model &md_, bool enable_all_variables = false);
+    ga_workspace(bool, const ga_workspace &gaw);
+    ga_workspace();
+    ~ga_workspace();
 
   };
 
@@ -387,14 +522,14 @@ namespace getfem {
   std::string ga_substitute(const std::string &expr,
                             const std::map<std::string, std::string> &dict);
 
-  inline std::string ga_subsitute(const std::string &expr,
+  inline std::string ga_substitute(const std::string &expr,
                                   const std::string &o1,const std::string &s1) {
     std::map<std::string, std::string> dict;
     dict[o1] = s1;
     return ga_substitute(expr, dict);
   }
 
-  inline std::string ga_subsitute(const std::string &expr,
+  inline std::string ga_substitute(const std::string &expr,
                                   const std::string &o1,const std::string &s1,
                                   const std::string &o2,const std::string &s2) {
     std::map<std::string, std::string> dict;
@@ -402,7 +537,7 @@ namespace getfem {
     return ga_substitute(expr, dict);
   }
 
-  inline std::string ga_subsitute(const std::string &expr,
+  inline std::string ga_substitute(const std::string &expr,
                                   const std::string &o1,const std::string &s1,
                                   const std::string &o2,const std::string &s2,
                                   const std::string &o3,const std::string &s3) {
@@ -411,7 +546,7 @@ namespace getfem {
     return ga_substitute(expr, dict);
   }
 
-  inline std::string ga_subsitute(const std::string &expr,
+  inline std::string ga_substitute(const std::string &expr,
                                   const std::string &o1,const std::string &s1,
                                   const std::string &o2,const std::string &s2,
                                   const std::string &o3,const std::string &s3,
@@ -490,13 +625,20 @@ namespace getfem {
    const mesh_region &rg_source=mesh_region::all_convexes(),
    size_type nbdof_ = size_type(-1));
 
+  void ga_interpolation_im_data
+  (ga_workspace &workspace, const im_data &imd, base_vector &result);
+
   void ga_interpolation_im_data
   (const getfem::model &md, const std::string &expr, const im_data &imd,
    base_vector &result, const mesh_region &rg=mesh_region::all_convexes());
 
-  void ga_interpolation_im_data
-  (ga_workspace &workspace, const im_data &imd, base_vector &result,
-   const mesh_region &rg=mesh_region::all_convexes());
+  void ga_interpolation_mesh_slice
+  (ga_workspace &workspace, const stored_mesh_slice &sl, base_vector &result);
+
+  void ga_interpolation_mesh_slice
+  (const getfem::model &md, const std::string &expr, const stored_mesh_slice &sl,
+   base_vector &result, const mesh_region &rg=mesh_region::all_convexes());
+
 
   //=========================================================================
   // Local projection functions
@@ -517,21 +659,51 @@ namespace getfem {
   // Interpolate transformations
   //=========================================================================
 
-  /** Add a transformation to the model `md` from mesh `source_mesh` to mesh
-      `target_mesh` given by the expression `expr` which corresponds to a
-      high-level generic assembly expression which may contains some
-      variable of the model. CAUTION: For the moment, the derivative of the
-      transformation with respect to the eventual variables used is not
-      taken into account in the model solve.
+  /** Add a transformation to a workspace `workspace` or a model `md` mapping
+      point in mesh `source_mesh` to mesh `target_mesh`, optionally restricted
+      to the region `target_region`. The transformation is defined by the
+      expression `expr`, which has to be in the high-level generic assembly
+      syntax and may contain some variables of the workspace/model.
+      CAUTION: For the moment, the derivative of the transformation with
+      respect to any of these variables is not taken into account in the model
+      solve.
   */
-  void add_interpolate_transformation_from_expression
-  (model &md, const std::string &transname, const mesh &source_mesh,
-   const mesh &target_mesh, const std::string &expr);
-
   void add_interpolate_transformation_from_expression
   (ga_workspace &workspace, const std::string &transname,
    const mesh &source_mesh, const mesh &target_mesh, const std::string &expr);
+  void add_interpolate_transformation_from_expression
+  (ga_workspace &workspace, const std::string &transname,
+   const mesh &source_mesh, const mesh &target_mesh,
+   size_type target_region, const std::string &expr);
+  void add_interpolate_transformation_from_expression
+  (model &md, const std::string &transname,
+   const mesh &source_mesh, const mesh &target_mesh, const std::string &expr);
+  void add_interpolate_transformation_from_expression
+  (model &md, const std::string &transname,
+   const mesh &source_mesh, const mesh &target_mesh,
+   size_type target_region, const std::string &expr);
 
+  /** Add a transformation to the workspace that creates an identity mapping
+      between two meshes in deformed state. Conceptually, it can be viewed
+      as a transformation from expression Xsource + Usource - Utarget,
+      except such an expression cannot be used directly in the transformation
+      from expression (function above), as Utarget needs to be interpolated
+      though an inversion of the transformation of the target domain.
+      Thread safe if added to thread local workspace.
+  */
+  void add_interpolate_transformation_on_deformed_domains
+  (ga_workspace &workspace, const std::string &transname,
+   const mesh &source_mesh, const std::string &source_displacements,
+   const mesh_region &source_region, const mesh &target_mesh,
+   const std::string &target_displacements, const mesh_region &target_region);
+
+  /** The same as above, but adding transformation to the model.
+  Note, this version is not thread safe.*/
+  void add_interpolate_transformation_on_deformed_domains
+  (model &md, const std::string &transname,
+   const mesh &source_mesh, const std::string &source_displacements,
+   const mesh_region &source_region, const mesh &target_mesh,
+   const std::string &target_displacements, const mesh_region &target_region);
 
   /** Create a new instance of a transformation corresponding to the
       interpolation on the neighbour element. Can only be applied to the
@@ -540,6 +712,37 @@ namespace getfem {
   */
   pinterpolate_transformation interpolate_transformation_neighbour_instance();
 
+  /* Add a special interpolation transformation which represents the identity
+     transformation but allows to evaluate the expression on another element
+     than the current element by polynomial extrapolation. It is used for
+     stabilization term in fictitious domain applications. the map elt_cor
+     list the element concerned by the transformation and associate them
+     to the element on which the extrapolation has to be made. If an element
+     is not listed in elt_cor the evaluation is just made on the current
+     element.
+  */
+  void add_element_extrapolation_transformation
+  (model &md, const std::string &name, const mesh &sm,
+   std::map<size_type, size_type> &elt_corr);
+
+  void add_element_extrapolation_transformation
+  (ga_workspace &workspace, const std::string &name, const mesh &sm,
+   std::map<size_type, size_type> &elt_corr);
+  
+  /* Change the correspondance map of an element extrapolation interpolate
+     transformation.
+  */
+  void set_element_extrapolation_correspondance
+  (model &md, const std::string &name,
+   std::map<size_type, size_type> &elt_corr);
+  
+  void set_element_extrapolation_correspondance
+  (ga_workspace &workspace, const std::string &name,
+   std::map<size_type, size_type> &elt_corr);
+    
+ 
+
+
 }  /* end of namespace getfem.                                             */
 
 
