@@ -1,7 +1,7 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
 
- Copyright (C) 2001-2016 Yves Renard, Julien Pommier
+ Copyright (C) 2001-2017 Yves Renard, Julien Pommier
 
  This file is a part of GetFEM++
 
@@ -40,7 +40,7 @@
 #define GETFEM_INTERPOLATION_H__
 
 #include "getfem_mesh_fem.h"
-#include "bgeot_geotrans_inv.h"
+#include "bgeot_torus.h"
 #include "dal_tree_sorted.h"
 #include "getfem_im_data.h"
 #include "getfem_torus.h"
@@ -582,18 +582,32 @@ namespace getfem {
                   "Target fem not convenient for interpolation");
     }
     /* initialisation of the mesh_trans_inv */
+    bool is_target_torus = dynamic_cast<const torus_mesh *>(&mf_target.linked_mesh());
     if (rg_target.id() == mesh_region::all_convexes().id()) {
       size_type nbpts = mf_target.nb_basic_dof() / qdim_t;
-      for (size_type i = 0; i < nbpts; ++i)
-        mti.add_point(mf_target.point_of_basic_dof(i * qdim_t));
-      interpolation(mf_source, mti, U, V, M, version, extrapolation);
+      for (size_type i = 0; i < nbpts; ++i){
+        if (is_target_torus){
+          auto p = mf_target.point_of_basic_dof(i * qdim_t);
+          p.resize(msh.dim());
+          mti.add_point(p);
+        }
+        else mti.add_point(mf_target.point_of_basic_dof(i * qdim_t));
+      }
     }
     else {
-      for (dal::bv_visitor_c dof(mf_target.basic_dof_on_region(rg_target)); !dof.finished(); ++dof)
-        if (dof % qdim_t == 0)
-          mti.add_point_with_id(mf_target.point_of_basic_dof(dof), dof/qdim_t);
-      interpolation(mf_source, mti, U, V, M, version, extrapolation, 0, rg_source);
+      for (dal::bv_visitor_c dof(mf_target.basic_dof_on_region(rg_target));
+	   !dof.finished(); ++dof)
+        if (dof % qdim_t == 0){
+          if (is_target_torus){
+            auto p = mf_target.point_of_basic_dof(dof);
+            p.resize(msh.dim());
+            mti.add_point_with_id(p, dof/qdim_t);
+          }
+          else
+	    mti.add_point_with_id(mf_target.point_of_basic_dof(dof),dof/qdim_t);
+        }  
     }
+    interpolation(mf_source, mti, U, V, M, version, extrapolation, 0,rg_source);
 
     if (version == 0)
       mf_target.reduce_vector(V, VV);
@@ -725,17 +739,23 @@ namespace getfem {
     // typedef typename gmm::linalg_traits<const VECT>::value_type T;
 
     dim_type qdim = mf_source.get_qdim();
-    GMM_ASSERT1(qdim == im_target.nb_tensor_elem(),
+    size_type nb_dof = mf_source.nb_dof();
+    size_type nb_basic_dof = mf_source.nb_basic_dof();
+    size_type nodal_data_size = gmm::vect_size(nodal_data);
+    dim_type data_qdim = nodal_data_size / nb_dof;
+
+    GMM_ASSERT1(data_qdim * mf_source.nb_dof() == nodal_data_size,
+                "Incompatible size of mesh fem " << mf_source.nb_dof() * data_qdim <<
+                " with the data " << nodal_data_size);
+
+    GMM_ASSERT1(qdim * data_qdim == im_target.nb_tensor_elem(),
                 "Incompatible size of qdim for mesh_fem " << qdim
                 << " and im_data " << im_target.nb_tensor_elem());
     GMM_ASSERT1(&mf_source.linked_mesh() == &im_target.linked_mesh(),
                 "mf_source and im_data do not share the same mesh.");
 
-    size_type nb_dof = mf_source.nb_dof();
-    size_type nb_basic_dof = mf_source.nb_basic_dof();
-
-    GMM_ASSERT1(nb_dof == gmm::vect_size(nodal_data),
-                "Provided nodal data size is " << gmm::vect_size(nodal_data)
+    GMM_ASSERT1(nb_dof * data_qdim == nodal_data_size,
+                "Provided nodal data size is " << nodal_data_size
                 << " but expecting vector size of " << nb_dof);
 
     size_type size_im_data = im_target.nb_index(use_im_data_filter)
@@ -744,7 +764,7 @@ namespace getfem {
                 "Provided im data size is " << gmm::vect_size(int_pt_data)
                 << " but expecting vector size of " << size_im_data);
 
-    VECT extended_nodal_data_((nb_dof != nb_basic_dof) ? nb_basic_dof : 0);
+    VECT extended_nodal_data_((nb_dof != nb_basic_dof) ? nb_basic_dof * data_qdim : 0);
     if (nb_dof != nb_basic_dof)
       mf_source.extend_vector(nodal_data, extended_nodal_data_);
     const VECT &extended_nodal_data = (nb_dof == nb_basic_dof) ? nodal_data : extended_nodal_data_;
@@ -783,7 +803,7 @@ namespace getfem {
         fem_interpolation_context ctx(pgt, pfp, size_type(-1), G, cv);
         for (size_type i = 0; i < nb_int_pts; ++i, ++int_pt_id) {
           ctx.set_ii(i);
-          ctx.pf()->interpolation(ctx, coeff, tensor_int_point.as_vector(), qdim);
+          ctx.pf()->interpolation(ctx, coeff, tensor_int_point.as_vector(), qdim * data_qdim);
           im_target.set_tensor(int_pt_data, int_pt_id, tensor_int_point);
         }
       }
@@ -798,7 +818,7 @@ namespace getfem {
           size_type i0 = pim->ind_first_point_on_face(f);
           for (size_type i = 0; i < nb_int_pts; ++i, ++int_pt_id) {
             ctx.set_ii(i+i0);
-            ctx.pf()->interpolation(ctx, coeff, tensor_int_point.as_vector(), qdim);
+            ctx.pf()->interpolation(ctx, coeff, tensor_int_point.as_vector(), qdim * data_qdim);
             im_target.set_tensor(int_pt_data, int_pt_id, tensor_int_point);
           }
         }
