@@ -90,6 +90,7 @@ namespace getfem {
   class mesher_half_space : public mesher_signed_distance {
     base_node x0; base_small_vector n; scalar_type xon;
   public:
+    mesher_half_space() = default;
     mesher_half_space(const base_node &x0_, const base_small_vector &n_)
       : x0(x0_), n(n_)
     { n /= gmm::vect_norm2(n); xon = gmm::vect_sp(x0, n); }
@@ -227,14 +228,14 @@ namespace getfem {
     std::vector<mesher_half_space> hfs;
   public:
     mesher_rectangle(base_node rmin_, base_node rmax_)
-      : rmin(rmin_), rmax(rmax_) {
+      : rmin(rmin_), rmax(rmax_), hfs(rmin.size()*2) {
       base_node n(rmin_.size());
       for (unsigned k = 0; k < rmin.size(); ++k) {
-	n[k] = 1.0;
-	hfs.push_back(mesher_half_space(rmin, n));
-	n[k] = -1.0;
-	hfs.push_back(mesher_half_space(rmax, n));
-	n[k] = 0.0;
+        n[k] = 1.0;
+        hfs[k*2] = mesher_half_space(rmin, n);
+        n[k] = -1.0;
+        hfs[k*2+1] = mesher_half_space(rmax, n);
+        n[k] = 0.0;
       }
     }
     bool bounding_box(base_node &bmin, base_node &bmax) const {
@@ -281,22 +282,21 @@ namespace getfem {
   { return std::make_shared<mesher_rectangle>(rmin, rmax); }
 
   class mesher_simplex_ref : public mesher_signed_distance {
-    // ajouter une rotation rigide, homothetie,  et translation ..
+    // To be added : rigid motion, dilatation ...
     std::vector<mesher_half_space> hfs;
     unsigned N;
     base_node org;
   public:
-    mesher_simplex_ref(unsigned N_) : N(N_) {
+    mesher_simplex_ref(unsigned N_) : hfs(N_+1), N(N_), org(N_) {
       base_node no(N);
-      org = no;
       for (unsigned k = 0; k < N; ++k) {
-	no[k] = scalar_type(1);
-	hfs.push_back(mesher_half_space(org, no));
-	no[k] = scalar_type(0);
+        no[k] = 1.0;
+        hfs[k] = mesher_half_space(org, no);
+        no[k] = 0.0;
       }
-      std::fill(org.begin(), org.end(), scalar_type(1)/scalar_type(N));
+      std::fill(org.begin(), org.end(), 1.0/N);
       no = -org;
-      hfs.push_back(mesher_half_space(org, no));
+      hfs[N] = mesher_half_space(org, no);
     }
     bool bounding_box(base_node &bmin, base_node &bmax) const {
       bmin.resize(N); bmax.resize(N);
@@ -338,27 +338,26 @@ namespace getfem {
 
 
   class mesher_prism_ref : public mesher_signed_distance {
-    // ajouter une rotation rigide, homothetie,  et translation ..
+    // To be added : rigid motion, dilatation ...
     std::vector<mesher_half_space> hfs;
     unsigned N;
     base_node org;
   public:
-    mesher_prism_ref(unsigned N_) : N(N_) {
+    mesher_prism_ref(unsigned N_) : hfs(N_+2), N(N_) {  
       base_node no(N);
       org = no;
       for (unsigned k = 0; k < N; ++k) {
-	no[k] = scalar_type(1);
-	hfs.push_back(mesher_half_space(org, no));
-	no[k] = scalar_type(0);
+        no[k] = 1.0;
+        hfs[k] = mesher_half_space(org, no);
+        no[k] = 0.0;
       }
-      no[N-1] = -scalar_type(1);
-      org[N-1] = scalar_type(1);
-      hfs.push_back(mesher_half_space(org, no));
-      std::fill(org.begin(), org.end(), scalar_type(1)/scalar_type(N));
-      org[N-1] = scalar_type(0);
+      no[N-1] = -1.0;
+      org[N-1] = 1.0;
+      hfs[N] = mesher_half_space(org, no);
+      std::fill(org.begin(), org.end(), 1.0/N);
+      org[N-1] = 0.0;
       no = -org;
-      hfs.push_back(mesher_half_space(org, no));
-      
+      hfs[N+1] = mesher_half_space(org, no);    
     }
     bool bounding_box(base_node &bmin, base_node &bmax) const {
       bmin.resize(N); bmax.resize(N);
@@ -1068,12 +1067,6 @@ namespace getfem {
 
   inline pmesher_signed_distance new_mesher_torus(scalar_type R, scalar_type r)
   { return std::make_shared<mesher_torus>(R,r); }
-
-
-  // interface with qhull
-  void delaunay(const std::vector<base_node> &pts,
-		gmm::dense_matrix<size_type>& simplexes);
-
   
   // mesher
   void build_mesh(mesh &m, const pmesher_signed_distance& dist_,
