@@ -1,7 +1,7 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
 
- Copyright (C) 2012-2016 Andriy Andreykiv
+ Copyright (C) 2012-2017 Andriy Andreykiv
 
  This file is a part of GetFEM++
 
@@ -43,11 +43,6 @@ This is the kernel of getfem.
 #include <vector>
 #include <algorithm>
 
-#ifdef GETFEM_HAVE_OPENMP
-#include <omp.h>
-#include <boost/thread.hpp>
-#endif
-
 #ifdef _WIN32
 #include <mbctype.h>
 #endif
@@ -57,6 +52,8 @@ This is the kernel of getfem.
 #include "gmm/gmm_std.h"
 #include "bgeot_config.h"
 #ifdef GETFEM_HAVE_OPENMP
+#include <omp.h>
+#include <boost/thread.hpp>
 #include <boost/shared_ptr.hpp>
 #include <boost/make_shared.hpp>
 #endif
@@ -158,9 +155,7 @@ namespace getfem
       }
     };
   public:
-    omp_distribute() : thread_values(num_threads()) {}
-    omp_distribute(const T& value) :
-      thread_values(num_threads(),value) {}
+
     template <class... Args>
     omp_distribute(Args&&... value){
       for (size_type i = 0; i < num_threads(); ++i) thread_values.emplace_back(value...);
@@ -372,7 +367,11 @@ namespace getfem
     template <typename Function, typename... Parameters>
     void run(Function f, Parameters... params)
     {
+#ifdef GETFEM_HAVE_OPENMP
       try { f(params...); } catch (...) { captureException(); }
+#else
+      f(params...);
+#endif
     }
 
     /**vector of pointers to caught exceptions*/
