@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2004-2016 Yves Renard, Konstantinos Poulios.
+ Copyright (C) 2004-2017 Yves Renard, Konstantinos Poulios.
 
  This file is a part of GetFEM++
 
@@ -318,7 +318,7 @@ namespace getfem {
         gmm::mult(grad_fc, gmm::transposed(base_ref_fc), grad_fc1);
         gmm::mult(G, grad_fc1, K);
         gmm::mult(gmm::transposed(K), K, CS);
-        gmm::lu_inverse(CS);
+        bgeot::lu_inverse(&(*(CS.begin())), P-1);
         gmm::mult(K, CS, B);
         gmm::mult(B, base_ref_fc, BB);
 
@@ -352,7 +352,7 @@ namespace getfem {
           pgt->poly_vector_grad(xp, grad_cv);
 
           base_matrix GG(N, nb_pts_cv);
-          vectors_to_base_matrix(GG, m.points_of_convex(cv));
+          m.points_of_convex(cv, GG);
 
           gmm::mult(GG, grad_cv, KK);
         }
@@ -416,7 +416,9 @@ namespace getfem {
         const mesh &mesh_m = cn_m->mf->linked_mesh();
         base_node slave_node = cn_s->mf->point_of_basic_dof(cn_s->dof);
         base_node master_node = cn_m->mf->point_of_basic_dof(cn_m->dof);
-        base_node un_sel(3), proj_node_sel(3), proj_node_ref_sel(3);
+        GMM_ASSERT1(slave_node.size() == qdim && master_node.size() == qdim,
+                    "Internal error");
+        base_node un_sel(qdim), proj_node_sel(qdim), proj_node_ref_sel(qdim);
         scalar_type is_in_min = 1e5;  //FIXME
         size_type cv_sel = 0;
         short_type fc_sel = 0;
@@ -424,7 +426,7 @@ namespace getfem {
         std::vector<short_type>::iterator fc;
         for (cv = cn_m->cvs.begin(), fc = cn_m->fcs.begin();
              cv != cn_m->cvs.end() && fc != cn_m->fcs.end(); cv++, fc++) {
-          base_node un(3), proj_node(3), proj_node_ref(3);
+          base_node un(qdim), proj_node(qdim), proj_node_ref(qdim);
           scalar_type is_in = projection_on_convex_face
             (mesh_m, *cv, *fc, master_node, slave_node, un, proj_node, proj_node_ref);
           if (is_in < is_in_min) {
@@ -439,8 +441,8 @@ namespace getfem {
         if (is_in_min < 0.05) {  //FIXME
           gap[row] = gmm::vect_sp(slave_node-proj_node_sel, un_sel);
 
-          base_node ut[3];
-          if (BT1) orthonormal_basis_to_unit_vec(d, un_sel, ut);
+	  std::vector<base_node> ut(d);
+          if (BT1) orthonormal_basis_to_unit_vec(d, un_sel, &(ut[0]));
 
           CONTACT_B_MATRIX *BN = 0;
           CONTACT_B_MATRIX *BT = 0;
@@ -473,7 +475,7 @@ namespace getfem {
           if (BN) {
             base_matrix G;
             base_matrix M(qdim, mf_disp->nb_basic_dof_of_element(cv_sel));
-            bgeot::vectors_to_base_matrix(G, mesh_m.points_of_convex(cv_sel));
+            mesh_m.points_of_convex(cv_sel, G);
             pfem pf = mf_disp->fem_of_element(cv_sel);
             bgeot::pgeometric_trans pgt = mesh_m.trans_of_convex(cv_sel);
             fem_interpolation_context
