@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2009-2015 Yves Renard
+ Copyright (C) 2009-2017 Yves Renard
 
  This file is a part of GetFEM++
 
@@ -120,7 +120,7 @@ namespace getfem {
   (model &md, const mesh_im &mim, const std::string &varname,
    const std::string &dataname,
    size_type region) {
-    pbrick pbr = new bilap_brick;
+    pbrick pbr = std::make_shared<bilap_brick>();
     model::termlist tl;
     tl.push_back(model::term_description(varname, varname, true));
     model::varnamelist dl(1, dataname);
@@ -132,7 +132,7 @@ namespace getfem {
   (model &md, const mesh_im &mim, const std::string &varname,
    const std::string &dataname1, const std::string &dataname2,
    size_type region) {
-    pbrick pbr = new bilap_brick;
+    pbrick pbr = std::make_shared<bilap_brick>();
     model::termlist tl;
     tl.push_back(model::term_description(varname, varname, true));
     model::varnamelist dl(1, dataname1);
@@ -240,7 +240,7 @@ namespace getfem {
       set_flags("Normal derivative source term", true /* is linear*/,
 		true /* is symmetric */, true /* is coercive */,
 		true /* is real */, true /* is complex */,
-		false /* compute each time */, false /* has a Neumann term */);
+		false /* compute each time */);
     }
 
 
@@ -249,7 +249,7 @@ namespace getfem {
   size_type add_normal_derivative_source_term_brick
   (model &md, const mesh_im &mim, const std::string &varname,
    const std::string &dataname, size_type region) {
-    pbrick pbr = new normal_derivative_source_term_brick;
+    pbrick pbr = std::make_shared<normal_derivative_source_term_brick>();
     model::termlist tl;
     tl.push_back(model::term_description(varname));
     model::varnamelist vdata(1, dataname);
@@ -277,13 +277,13 @@ namespace getfem {
 					size_type region,
 					build_version) const {
       GMM_ASSERT1(vecl.size() == 1,
-		  "Kirchoff Love source term brick has one and only "
+		  "Kirchhoff Love source term brick has one and only "
 		  "one term");
       GMM_ASSERT1(mims.size() == 1,
-		  "Kirchoff Love source term brick need one and only "
+		  "Kirchhoff Love source term brick need one and only "
 		  "one mesh_im");
       GMM_ASSERT1(vl.size() == 1 && dl.size() == 2,
-		  "Wrong number of variables for Kirchoff Love "
+		  "Wrong number of variables for Kirchhoff Love "
 		  "source term brick");
 
       const mesh_fem &mf_u = md.mesh_fem_of_variable(vl[0]);
@@ -299,19 +299,19 @@ namespace getfem {
       size_type s = gmm::vect_size(A);
       if (mf_dataA) s = s * mf_dataA->get_qdim() / mf_dataA->nb_dof();
       GMM_ASSERT1(mf_u.get_qdim() == 1 && s == N*N,
-		  dl[0] << ": bad format of Kirchoff Love Neumann term "
+		  dl[0] << ": bad format of Kirchhoff Love Neumann term "
 		  "data. Detected dimension is " << s << " should be "
 		  << N*N);
 
       s = gmm::vect_size(B);
       if (mf_dataB) s = s * mf_dataB->get_qdim() / mf_dataB->nb_dof();
       GMM_ASSERT1(s == N,
-		  dl[0] << ": bad format of Kirchoff Love Neumann term "
+		  dl[0] << ": bad format of Kirchhoff Love Neumann term "
 		  "data. Detected dimension is " << s << " should be "
 		  << N);
 
 
-      GMM_TRACE2("Kirchoff Love Neumann term assembly");
+      GMM_TRACE2("Kirchhoff Love Neumann term assembly");
       if (mf_dataA)
 	asm_neumann_KL_term(vecl[0], mim, mf_u, *mf_dataA, A, B, rg);
       else
@@ -320,20 +320,20 @@ namespace getfem {
     }
 
     KL_source_term_brick(void) {
-      set_flags("Kirchoff Love Neumann term", true /* is linear*/,
+      set_flags("Kirchhoff Love Neumann term", true /* is linear*/,
 		true /* is symmetric */, true /* is coercive */,
 		true /* is real */, false /* is complex */,
-		false /* compute each time */, false /* has a Neumann term */);
+		false /* compute each time */);
     }
 
 
   };
 
-  size_type add_Kirchoff_Love_Neumann_term_brick
+  size_type add_Kirchhoff_Love_Neumann_term_brick
   (model &md, const mesh_im &mim, const std::string &varname,
    const std::string &dataname1, const std::string &dataname2,
    size_type region) {
-    pbrick pbr = new KL_source_term_brick;
+    pbrick pbr = std::make_shared<KL_source_term_brick>();
     model::termlist tl;
     tl.push_back(model::term_description(varname));
     model::varnamelist vdata(1, dataname1);
@@ -470,9 +470,11 @@ namespace getfem {
 	      asm_source_term(*R, mim, mf_mult, *mf_data, *A, rg);
 	  }
 	  else {
-	    asm_real_or_complex_1_param
-	      (*R, mim, mf_mult, *mf_data, *A, rg,
-	       "R=data(#2); V(#1)+=comp(Base(#1).Grad(#2).Normal())(:,i,j,j).R(i)");
+	    asm_real_or_complex_1_param_vec
+	      (*R, mim, mf_mult, mf_data, *A, rg, "(Grad_A.Normal)*Test_u");
+	    // asm_real_or_complex_1_param
+	    //   (*R, mim, mf_mult, *mf_data, *A, rg,
+	    //    "R=data(#2); V(#1)+=comp(Base(#1).Grad(#2).Normal())(:,i,j,j).R(i)");
 	  }
 	} else {
 	  GMM_ASSERT1(!R_must_be_derivated, "Incoherent situation");
@@ -580,9 +582,11 @@ namespace getfem {
 	      asm_source_term(*R, mim, mf_mult, *mf_data, *A, rg);
 	  }
 	  else {
-	    asm_real_or_complex_1_param
-	      (*R, mim, mf_mult, *mf_data, *A, rg,
-	       "R=data(#2); V(#1)+=comp(Base(#1).Grad(#2).Normal())(:,i,j,j).R(i)");
+	    asm_real_or_complex_1_param_vec
+	      (*R, mim, mf_mult, mf_data, *A, rg, "(Grad_A.Normal)*Test_u");
+	    // asm_real_or_complex_1_param
+	    //   (*R, mim, mf_mult, *mf_data, *A, rg,
+	    //    "R=data(#2); V(#1)+=comp(Base(#1).Grad(#2).Normal())(:,i,j,j).R(i)");
 	  }
 	} else {
 	  GMM_ASSERT1(!R_must_be_derivated, "Incoherent situation");
@@ -608,7 +612,7 @@ namespace getfem {
 		true /* is linear*/,
 		true /* is symmetric */, penalized /* is coercive */,
 		true /* is real */, true /* is complex */,
-		false /* compute each time */, false /* has a Neumann term */);
+		false /* compute each time */);
     }
   };
 
@@ -616,7 +620,7 @@ namespace getfem {
   (model &md, const mesh_im &mim, const std::string &varname,
    const std::string &multname, size_type region,
    const std::string &dataname, bool R_must_be_derivated) {
-    pbrick pbr = new normal_derivative_Dirichlet_condition_brick(false, R_must_be_derivated);
+    pbrick pbr = std::make_shared<normal_derivative_Dirichlet_condition_brick>(false, R_must_be_derivated);
     model::termlist tl;
     tl.push_back(model::term_description(multname, varname, true));
     model::varnamelist vl(1, varname);
@@ -658,7 +662,7 @@ namespace getfem {
       md.set_complex_variable(coeffname)[0] = penalisation_coeff;
     else
       md.set_real_variable(coeffname)[0] = penalisation_coeff;
-    pbrick pbr = new normal_derivative_Dirichlet_condition_brick(true, R_must_be_derivated);
+    pbrick pbr = std::make_shared<normal_derivative_Dirichlet_condition_brick>(true, R_must_be_derivated);
     model::termlist tl;
     tl.push_back(model::term_description(varname, varname, true));
     model::varnamelist vl(1, varname);
