@@ -28,43 +28,45 @@
 namespace getfem {
 
   void mesh_fem::update_from_context() const {
-    for (dal::bv_visitor i(fe_convex); !i.finished(); ++i) {
-      if (linked_mesh_->convex_index().is_in(i)) {
-        if (v_num_update < linked_mesh_->convex_version_number(i)) {
+    for (dal::bv_visitor cv(fe_convex); !cv.finished(); ++cv) {
+      if (linked_mesh_->convex_index().is_in(cv)) {
+        if (v_num_update < linked_mesh_->convex_version_number(cv)) {
           if (auto_add_elt_pf != 0)
             const_cast<mesh_fem *>(this)
-              ->set_finite_element(i, auto_add_elt_pf);
+              ->set_finite_element(cv, auto_add_elt_pf);
           else if (auto_add_elt_K != dim_type(-1)) {
             if (auto_add_elt_disc)
               const_cast<mesh_fem *>(this)
-                ->set_classical_discontinuous_finite_element(i,auto_add_elt_K,
-                                                         auto_add_elt_alpha);
+                ->set_classical_discontinuous_finite_element
+                  (cv, auto_add_elt_K, auto_add_elt_alpha,
+                   auto_add_elt_complete);
             else
               const_cast<mesh_fem *>(this)
-                ->set_classical_finite_element(i, auto_add_elt_K);
+                ->set_classical_finite_element(cv, auto_add_elt_K,
+                                               auto_add_elt_complete);
           }
           else
-            const_cast<mesh_fem *>(this)
-              ->set_finite_element(i, 0);
+            const_cast<mesh_fem *>(this)->set_finite_element(cv, 0);
         }
       }
-      else const_cast<mesh_fem *>(this)->set_finite_element(i, 0);
+      else const_cast<mesh_fem *>(this)->set_finite_element(cv, 0);
     }
-    for (dal::bv_visitor i(linked_mesh_->convex_index());
-         !i.finished(); ++i) {
-      if (!fe_convex.is_in(i)
-          && v_num_update < linked_mesh_->convex_version_number(i)) {
+    for (dal::bv_visitor cv(linked_mesh_->convex_index());
+         !cv.finished(); ++cv) {
+      if (!fe_convex.is_in(cv)
+          && v_num_update < linked_mesh_->convex_version_number(cv)) {
         if (auto_add_elt_pf != 0)
           const_cast<mesh_fem *>(this)
-            ->set_finite_element(i, auto_add_elt_pf);
+            ->set_finite_element(cv, auto_add_elt_pf);
         else if (auto_add_elt_K != dim_type(-1)) {
           if (auto_add_elt_disc)
             const_cast<mesh_fem *>(this)
-              ->set_classical_discontinuous_finite_element(i, auto_add_elt_K,
-                                                           auto_add_elt_alpha);
+              ->set_classical_discontinuous_finite_element
+                (cv, auto_add_elt_K, auto_add_elt_alpha, auto_add_elt_complete);
           else
             const_cast<mesh_fem *>(this)
-              ->set_classical_finite_element(i, auto_add_elt_K);
+              ->set_classical_finite_element(cv, auto_add_elt_K,
+                                             auto_add_elt_complete);
         }
       }
     }
@@ -143,20 +145,20 @@ namespace getfem {
                   " and target_dim " << int(pf->target_dim()) << " of " <<
                   name_of_fem(pf));
 
-      
+
       if (cv == f_elems.size()) {
-	f_elems.push_back(pf);
-	fe_convex.add(cv);
-	dof_enumeration_made = false;
+        f_elems.push_back(pf);
+        fe_convex.add(cv);
+        dof_enumeration_made = false;
         touch(); v_num = act_counter();
       } else {
-	if (cv > f_elems.size()) f_elems.resize(cv+1);
-	if (!fe_convex.is_in(cv) || f_elems[cv] != pf) {
-	  fe_convex.add(cv);
-	  f_elems[cv] = pf;
-	  dof_enumeration_made = false;
-	  touch(); v_num = act_counter();
-	}
+        if (cv > f_elems.size()) f_elems.resize(cv+1);
+        if (!fe_convex.is_in(cv) || f_elems[cv] != pf) {
+          fe_convex.add(cv);
+          f_elems[cv] = pf;
+          dof_enumeration_made = false;
+          touch(); v_num = act_counter();
+        }
       }
     }
   }
@@ -170,46 +172,51 @@ namespace getfem {
   { set_finite_element(linked_mesh().convex_index(), ppf); set_auto_add(ppf); }
 
   void mesh_fem::set_classical_finite_element(size_type cv,
-                                              dim_type fem_degree) {
+                                              dim_type fem_degree,
+                                              bool complete) {
     pfem pf = getfem::classical_fem(linked_mesh().trans_of_convex(cv),
-                                    fem_degree);
+                                    fem_degree, complete);
     set_finite_element(cv, pf);
   }
 
   void mesh_fem::set_classical_finite_element(const dal::bit_vector &cvs,
-                                              dim_type fem_degree) {
+                                              dim_type fem_degree,
+                                              bool complete) {
     for (dal::bv_visitor cv(cvs); !cv.finished(); ++cv) {
       pfem pf = getfem::classical_fem(linked_mesh().trans_of_convex(cv),
-                                      fem_degree);
+                                      fem_degree, complete);
       set_finite_element(cv, pf);
     }
   }
 
-  void mesh_fem::set_classical_finite_element(dim_type fem_degree) {
-    set_classical_finite_element(linked_mesh().convex_index(), fem_degree);
+  void mesh_fem::set_classical_finite_element(dim_type fem_degree,
+                                              bool complete) {
+    set_classical_finite_element(linked_mesh().convex_index(), fem_degree,
+                                 complete);
     set_auto_add(fem_degree, false);
   }
 
   void mesh_fem::set_classical_discontinuous_finite_element
-  (size_type cv, dim_type fem_degree, scalar_type alpha) {
+  (size_type cv, dim_type fem_degree, scalar_type alpha, bool complete) {
     pfem pf = getfem::classical_discontinuous_fem
-      (linked_mesh().trans_of_convex(cv), fem_degree, alpha);
+      (linked_mesh().trans_of_convex(cv), fem_degree, alpha, complete);
     set_finite_element(cv, pf);
   }
 
   void mesh_fem::set_classical_discontinuous_finite_element
-  (const dal::bit_vector &cvs, dim_type fem_degree, scalar_type alpha) {
+  (const dal::bit_vector &cvs, dim_type fem_degree, scalar_type alpha,
+   bool complete) {
     for (dal::bv_visitor cv(cvs); !cv.finished(); ++cv) {
       pfem pf = getfem::classical_discontinuous_fem
-        (linked_mesh().trans_of_convex(cv), fem_degree, alpha);
+        (linked_mesh().trans_of_convex(cv), fem_degree, alpha, complete);
       set_finite_element(cv, pf);
     }
   }
 
   void mesh_fem::set_classical_discontinuous_finite_element
-  (dim_type fem_degree, scalar_type alpha) {
-    set_classical_discontinuous_finite_element(linked_mesh().convex_index(),
-                                               fem_degree,alpha);
+  (dim_type fem_degree, scalar_type alpha, bool complete) {
+    set_classical_discontinuous_finite_element
+      (linked_mesh().convex_index(), fem_degree, alpha, complete);
     set_auto_add(fem_degree, true, alpha);
   }
 
@@ -335,7 +342,7 @@ namespace getfem {
     // Information for global dof
     dal::bit_vector encountered_global_dof, processed_elt;
     dal::dynamic_array<size_type> ind_global_dof;
-    
+
     // Auxilliary variables
     std::vector<size_type> itab;
     base_node P(linked_mesh().dim());
@@ -349,25 +356,25 @@ namespace getfem {
     bgeot::pgeotrans_precomp pgp = 0;
 
     for (dal::bv_visitor cv(linked_mesh().convex_index());
-      	 !cv.finished(); ++cv) {
+         !cv.finished(); ++cv) {
       if (fe_convex.is_in(cv)) {
-	gmm::copy(linked_mesh().points_of_convex(cv)[0], bmin);
-	gmm::copy(bmin, bmax);
-	for (size_type i = 0; i <  linked_mesh().nb_points_of_convex(cv); ++i) {
-	  const base_node &pt = linked_mesh().points_of_convex(cv)[i];
-	  for (size_type d = 1; d < bmin.size(); ++d) {
-	    bmin[d] = std::min(bmin[d], pt[d]);
-	    bmax[d] = std::max(bmax[d], pt[d]);
-	  }
-	}	
-	elt_car_sizes[cv] = gmm::vect_dist2(bmin, bmax);
+        gmm::copy(linked_mesh().points_of_convex(cv)[0], bmin);
+        gmm::copy(bmin, bmax);
+        for (size_type i = 0; i < linked_mesh().nb_points_of_convex(cv); ++i) {
+          const base_node &pt = linked_mesh().points_of_convex(cv)[i];
+          for (size_type d = 1; d < bmin.size(); ++d) {
+            bmin[d] = std::min(bmin[d], pt[d]);
+            bmax[d] = std::max(bmax[d], pt[d]);
+          }
+        }
+        elt_car_sizes[cv] = gmm::vect_dist2_sqr(bmin, bmax);
       }
     }
 
     dal::bit_vector cv_done;
-    
+
     for (dal::bv_visitor cv(linked_mesh().convex_index());
-	 !cv.finished(); ++cv) { // Loop on elements
+         !cv.finished(); ++cv) { // Loop on elements
       if (!fe_convex.is_in(cv)) continue;
       pfem pf = fem_of_element(cv);
       if (pf != first_pf) is_uniform_ = false;
@@ -400,34 +407,34 @@ namespace getfem {
           pgp->transform(linked_mesh().points_of_convex(cv), i, P);
           size_type idof = nbdof;
 
-	  if (dof_nodes[cv].nb_points() > 0) {
-	    scalar_type dist = dof_nodes[cv].nearest_neighbor(ipt, P);
-	    if (gmm::abs(dist) <= 1e-6*elt_car_sizes[cv]) {
-	      fd.ind_node=ipt.i;
-	      auto it = dof_sorts[cv].find(fd);
-	      if (it != dof_sorts[cv].end()) idof = it->second;
-	    }
-	  }
-	  
+          if (dof_nodes[cv].nb_points() > 0) {
+            scalar_type dist = dof_nodes[cv].nearest_neighbor(ipt, P);
+            if (gmm::abs(dist) <= 1e-6*elt_car_sizes[cv]) {
+              fd.ind_node=ipt.i;
+              auto it = dof_sorts[cv].find(fd);
+              if (it != dof_sorts[cv].end()) idof = it->second;
+            }
+          }
+
           if (idof == nbdof) {
-	    nbdof += Qdim / pf->target_dim();
+            nbdof += Qdim / pf->target_dim();
 
-	    linked_mesh().neighbours_of_convex(cv, pf->faces_of_dof(cv, i), s);
-	    for (size_type ncv : s) { // For each unscanned neighbour
-	      if (!cv_done[ncv] && fe_convex.is_in(ncv)) { // add the dof
+            linked_mesh().neighbours_of_convex(cv, pf->faces_of_dof(cv, i), s);
+            for (size_type ncv : s) { // For each unscanned neighbour
+              if (!cv_done[ncv] && fe_convex.is_in(ncv)) { // add the dof
 
-		fd.ind_node = size_type(-1);
-		if (dof_nodes[ncv].nb_points() > 0) {
-		  scalar_type dist = dof_nodes[ncv].nearest_neighbor(ipt, P);
-		  if (gmm::abs(dist) <= 1e-6*elt_car_sizes[ncv])
-		    fd.ind_node=ipt.i;
-		}
-		if (fd.ind_node == size_type(-1))
-		  fd.ind_node = dof_nodes[ncv].add_point(P);
-		dof_sorts[ncv][fd] = idof;
-	      }
-	    }
-	  }
+                fd.ind_node = size_type(-1);
+                if (dof_nodes[ncv].nb_points() > 0) {
+                  scalar_type dist = dof_nodes[ncv].nearest_neighbor(ipt, P);
+                  if (gmm::abs(dist) <= 1e-6*elt_car_sizes[ncv])
+                    fd.ind_node=ipt.i;
+                }
+                if (fd.ind_node == size_type(-1))
+                  fd.ind_node = dof_nodes[ncv].add_point(P);
+                dof_sorts[ncv][fd] = idof;
+              }
+            }
+          }
           itab[i] = idof;
         }
       }
@@ -500,6 +507,7 @@ namespace getfem {
     auto_add_elt_pf = mf.auto_add_elt_pf;
     auto_add_elt_K = mf.auto_add_elt_K;
     auto_add_elt_disc = mf.auto_add_elt_disc;
+    auto_add_elt_complete = mf.auto_add_elt_complete;
     auto_add_elt_alpha = mf.auto_add_elt_alpha;
     mi = mf.mi;
     dof_partition = mf.dof_partition;
@@ -571,8 +579,8 @@ namespace getfem {
         } else if (bgeot::casecmp(tmp, "DOF_ENUMERATION") == 0) {
           dal::bit_vector doflst;
           dof_structure.clear(); dof_enumeration_made = false;
-	  is_uniform_ = true;
-	  size_type nbdof_unif = size_type(-1);
+          is_uniform_ = true;
+          size_type nbdof_unif = size_type(-1);
           touch(); v_num = act_counter();
           while (true) {
             bgeot::get_token(ist, tmp);
@@ -585,11 +593,11 @@ namespace getfem {
             std::vector<size_type> tab;
             if (convex_index().is_in(ic) && tmp.size() &&
                 isdigit(tmp[0]) && tmp2 == ":") {
-	      size_type nbd = nb_basic_dof_of_element(ic);
-	      if (nbdof_unif == size_type(-1))
-		nbdof_unif = nbd;
-	      else if (nbdof_unif != nbd)
-		is_uniform_ = false;
+              size_type nbd = nb_basic_dof_of_element(ic);
+              if (nbdof_unif == size_type(-1))
+                nbdof_unif = nbd;
+              else if (nbdof_unif != nbd)
+                is_uniform_ = false;
               tab.resize(nb_basic_dof_of_element(ic));
               for (size_type i=0; i < fem_of_element(ic)->nb_dof(ic);
                    i++) {
@@ -787,8 +795,9 @@ namespace getfem {
   struct mf__key_ : public context_dependencies {
     const mesh *pmsh;
     dim_type order, qdim;
-    mf__key_(const mesh &msh, dim_type o, dim_type q)
-      : pmsh(&msh), order(o), qdim(q)
+    bool complete;
+    mf__key_(const mesh &msh, dim_type o, dim_type q, bool complete_)
+      : pmsh(&msh), order(o), qdim(q), complete(complete_)
     { add_dependency(msh); }
     bool operator <(const mf__key_ &a) const {
       if (pmsh < a.pmsh) return true;
@@ -796,11 +805,16 @@ namespace getfem {
       else if (order < a.order) return true;
       else if (order > a.order) return false;
       else if (qdim < a.qdim) return true;
+      else if (qdim > a.qdim) return false;
+      else if (complete < a.complete) return true;
       return false;
     }
     void update_from_context() const {}
     mf__key_(const mf__key_ &mfk) : context_dependencies( ) {
-      pmsh = mfk.pmsh; order = mfk.order; qdim = mfk.qdim;
+      pmsh = mfk.pmsh;
+      order = mfk.order;
+      qdim = mfk.qdim;
+      complete = mfk.complete;
       add_dependency(*pmsh);
     }
   private :
@@ -817,7 +831,8 @@ namespace getfem {
 
   public :
 
-    const mesh_fem &operator()(const mesh &msh, dim_type o, dim_type qdim) {
+    const mesh_fem &operator()(const mesh &msh, dim_type o, dim_type qdim,
+                               bool complete=false) {
       mesh_fem_tab::iterator itt = mfs.begin(), itn = mfs.begin();
       if (itn != mfs.end()) itn++;
       while (itt != mfs.end()) {
@@ -827,7 +842,7 @@ namespace getfem {
         if (itn != mfs.end()) itn++;
       }
 
-      mf__key_ key(msh, o, qdim);
+      mf__key_ key(msh, o, qdim, complete);
       mesh_fem_tab::iterator it = mfs.find(key);
       assert(it == mfs.end() || it->second->is_context_valid());
 
@@ -845,10 +860,11 @@ namespace getfem {
   };
 
   const mesh_fem &classical_mesh_fem(const mesh &msh,
-                                     dim_type order, dim_type qdim) {
+                                     dim_type order, dim_type qdim,
+                                     bool complete) {
     classical_mesh_fem_pool &pool
       = dal::singleton<classical_mesh_fem_pool>::instance();
-    return pool(msh, order, qdim);
+    return pool(msh, order, qdim, complete);
   }
 
   struct dummy_mesh_fem_ {
