@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 1999-2016 Yves Renard
+ Copyright (C) 1999-2017 Yves Renard
 
  This file is a part of GetFEM++
 
@@ -29,7 +29,7 @@ namespace getfem {
     dim_ = cvr->structure()->dim();
     is_equiv = !smart_global_dof_linking_;
     real_element_defined = true;
-    is_polycomp = is_pol = is_lag = false;
+    is_polycomp = is_pol = is_lag = is_standard_fem = false;
     es_degree = 5; /* humm ... */
     ntarget_dim = 1;
 
@@ -215,10 +215,7 @@ namespace getfem {
   
   void fem_sum::real_hess_base_value(const fem_interpolation_context &c,
                                      base_tensor &t, bool withM) const {
-    bgeot::multi_index mi(4);
-    mi[3] = mi[2] = short_type(c.N()); mi[1] = target_dim();
-    mi[0] = short_type(nb_dof(0));
-    t.adjust_sizes(mi);
+    t.adjust_sizes(nb_dof(0), target_dim(), gmm::sqr(c.N()));
     base_tensor::iterator it = t.begin(), itf;
     
     fem_interpolation_context c0 = c;
@@ -231,16 +228,12 @@ namespace getfem {
       c0.hess_base_value(hess_e[k]);
     }
 
-    for (dim_type j = 0; j < c.N() ; ++j) {
-      for (dim_type k = 0; k < c.N() ; ++k) {
-        for (dim_type q = 0; q < target_dim(); ++q) {
-          for (size_type f = 0; f < pfems.size(); ++f) {
-            itf = hess_e[f].begin()
-              + ((j * c.N() + k) * target_dim() + q) * pfems[f]->nb_dof(cv); 
-            for (size_type i = 0; i < pfems[f]->nb_dof(cv); ++i)
-              *it++ = *itf++;
-          }
-        }
+    dim_type NNdim = dim_type(gmm::sqr(c.N())*target_dim());
+    for (dim_type jkq = 0; jkq < NNdim ; ++jkq) {
+      for (size_type f = 0; f < pfems.size(); ++f) {
+        itf = hess_e[f].begin() + (jkq * pfems[f]->nb_dof(cv)); 
+        for (size_type i = 0; i < pfems[f]->nb_dof(cv); ++i)
+          *it++ = *itf++;
       }
     }
     assert(it == t.end());
