@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2004-2016 Julien Pommier, Yves Renard
+ Copyright (C) 2004-2017 Julien Pommier, Yves Renard
 
  This file is a part of GetFEM++
 
@@ -109,7 +109,7 @@ namespace getfem {
     // cout << "nbco = " << nbco << endl;
     std::vector<const mesher_signed_distance*> ls(nbco);
     std::vector<scalar_type> d(nbco), v(nbco);
-    std::vector<int> ipvt(nbco);
+    gmm::lapack_ipvt ipvt(nbco);
     gmm::col_matrix<base_node> G(N, nbco);
     gmm::dense_matrix<scalar_type> H(nbco, nbco);
     base_small_vector dd(N);
@@ -269,7 +269,7 @@ namespace getfem {
     gmm::dense_matrix<T> B(n,n), C(n,n);
     gmm::mult(gmm::transposed(M), M, B);
     R trB = gmm::mat_trace(B);
-    gmm::lu_inverse(B);
+    bgeot::lu_inverse(&(*(B.begin())), n);
     R trBinv = gmm::mat_trace(B);
     gmm::mult(B,B,C);
     gmm::mult(gmm::scaled(M, T(-2)*trB), C, G);
@@ -356,7 +356,7 @@ namespace getfem {
             bgeot::equilateral_simplex_of_reference(dim_type(N))->points());
       gmm::mult(G, bgeot::geotrans_precomp(pgt, pgt->convex_ref()->pspt(),
 					   0)->grad(0), W);
-      gmm::lu_inverse(W);
+      bgeot::lu_inverse(&(*(W.begin())), N);
       do_build_mesh(m,fixed_points);
     }
 
@@ -384,7 +384,7 @@ namespace getfem {
           }
         }
         gmm::mult(S,W,SW);
-        if (gmm::lu_det(SW) < 1E-16) cost += 1e30;
+        if (bgeot::lu_det(&(*(SW.begin())), N) < 1E-16) cost += 1e30;
         else {
           scalar_type qual = gmm::Frobenius_condition_number_sqr(SW);
           cost += qual;
@@ -458,7 +458,7 @@ namespace getfem {
             S(k,j) = X[t(j+1,i)*N+k] - X[t(0,i)*N+k];
           }
         }
-        if (gmm::lu_det(S) < 0) {
+        if (bgeot::lu_det(&(*(S.begin())), N) < 0) {
           std::swap(t(0,i), t(1,i));
           for (size_type j=0; j < N; ++j) {
             for (size_type k=0; k < N; ++k) {
@@ -466,7 +466,7 @@ namespace getfem {
             }
           }
         }
-        if (noisy > 0 && gmm::abs(lu_det(S)) < 1e-10)
+        if (noisy > 0 && gmm::abs(bgeot::lu_det(&(*(S.begin())), N)) < 1e-10)
           cout << "Element " << i << " is very bad, det = "
                << gmm::abs(lu_det(S)) << "\n";
         gmm::mult(S,W,SW);
@@ -1171,7 +1171,7 @@ namespace getfem {
         cout << "NEW DELAUNAY, running on " << pts.size() << " points\n";
       size_type nbpt = pts.size();
       add_point_hull();
-      delaunay(pts, t);
+      bgeot::qhull_delaunay(pts, t);
       pts.resize(nbpt);
       if (noisy > 1) cout << "number of elements before selection = "
                           << gmm::mat_ncols(t) << "\n";
@@ -1308,8 +1308,8 @@ namespace getfem {
 
           control_mesh_surface();
           size_type nbpt = pts.size();
-          add_point_hull();          
-          delaunay(pts, t);
+          add_point_hull();
+          bgeot::qhull_delaunay(pts, t);
           pts.resize(nbpt);
           select_elements((prefind == 3) ? 0 : 1);
           suppress_flat_boundary_elements();
@@ -1343,7 +1343,7 @@ namespace getfem {
             control_mesh_surface();
             nbpt = pts.size();
             add_point_hull();
-            delaunay(pts, t);
+            bgeot::qhull_delaunay(pts, t);
             pts.resize(nbpt);
             select_elements((prefind == 3) ? 0 : 1);
             suppress_flat_boundary_elements();
@@ -1426,88 +1426,5 @@ namespace getfem {
               iter_max, prefind, dist_point_hull, boundary_threshold_flatness);
   }
   
-
-  // ******************************************************************
-  //    Interface with qhull
-  // ******************************************************************
-
-# ifndef GETFEM_HAVE_LIBQHULL_QHULL_A_H
-  void delaunay(const std::vector<base_node> &,
-                gmm::dense_matrix<size_type>&) {
-    GMM_ASSERT1(false, "Qhull header files not installed. "
-                "Install qhull library and reinstall GetFEM++ library.");
-  }
-# else
-
-  extern "C" {
-// #ifdef _MSC_VER
-# include <libqhull/qhull_a.h>
-// #else
-// # include <qhull/qhull.h>
-// # include <qhull/mem.h>
-// # include <qhull/qset.h>
-// # include <qhull/geom.h>
-// # include <qhull/merge.h>
-// # include <qhull/poly.h>
-// # include <qhull/io.h>
-// # include <qhull/stat.h>
-// #endif
-}
-
-  void delaunay(const std::vector<base_node> &pts,
-                gmm::dense_matrix<size_type>& simplexes) {
-    // cout << "running delaunay with " << pts.size() << " points\n";
-    size_type dim = pts[0].size();   /* points dimension.           */
-    if (pts.size() <= dim) { gmm::resize(simplexes, dim+1, 0); return; }
-    if (pts.size() == dim+1) {
-      gmm::resize(simplexes, dim+1, 1);
-      for (size_type i=0; i <= dim; ++i) simplexes(i, 0) = i;
-      return;
-    }
-    std::vector<coordT> Pts(dim * pts.size());
-    for (size_type i=0; i < pts.size(); ++i)
-      gmm::copy(pts[i], gmm::sub_vector(Pts, gmm::sub_interval(i*dim, dim)));
-    boolT ismalloc=0;  /* True if qhull should free points in
-                        * qh_freeqhull() or reallocation      */
-    /* Be Aware: option QJ could destabilizate all, it can break everything. */
-    /* option Qbb -> QbB (????) */
-    /* option flags for qhull, see qh_opt.htm */
-    char flags[]= "qhull QJ d Qbb Pp T0"; //QJ s i TO";//"qhull Tv";
-    FILE *outfile= 0;    /* output from qh_produce_output()
-                          *  use NULL to skip qh_produce_output() */ 
-    FILE *errfile= stderr;    /* error messages from qhull code */ 
-    int exitcode;             /* 0 if no error from qhull */
-    facetT *facet;                  /* set by FORALLfacets */
-    int curlong, totlong;          /* memory remaining after qh_memfreeshort */
-    vertexT *vertex, **vertexp;
-    exitcode = qh_new_qhull (int(dim), int(pts.size()), &Pts[0], ismalloc,
-                             flags, outfile, errfile);
-    if (!exitcode) { /* if no error */ 
-      size_type nbf=0;
-      FORALLfacets { if (!facet->upperdelaunay) nbf++; }
-      gmm::resize(simplexes, dim+1, nbf);
-        /* 'qh facet_list' contains the convex hull */
-      nbf=0;
-      FORALLfacets {
-        if (!facet->upperdelaunay) {
-          size_type s=0;
-          FOREACHvertex_(facet->vertices) {
-            assert(s < (unsigned)(dim+1));
-            simplexes(s++,nbf) = qh_pointid(vertex->point);
-          }
-          nbf++;
-        }
-      }
-    }
-    qh_freeqhull(!qh_ALL);
-    qh_memfreeshort (&curlong, &totlong);
-    if (curlong || totlong)
-      cerr << "qhull internal warning (main): did not free " << totlong << 
-        " bytes of long memory (" << curlong << " pieces)\n";
-
-  }
-
-#endif
-
 }
 
