@@ -298,6 +298,7 @@ namespace getfem {
       scalar_type no = ::sqrt(gmm::vect_norm2_sqr(t.as_vector())+gmm::sqr(eps));
       scalar_type no3 = no*no*no;
 
+      gmm::clear(result.as_vector());
       for (size_type i = 0; i < N; ++i) {
         result[i*N+i] += scalar_type(1)/no;
         for (size_type j = 0; j < N; ++j)
@@ -427,6 +428,7 @@ namespace getfem {
 
       switch (nder) {
       case 1:
+        gmm::clear(result.as_vector());
         for (size_type i = 0; i < N; ++i) {
           result[i*N+i] += scalar_type(1)/no;
           for (size_type j = 0; j < N; ++j)
@@ -768,8 +770,8 @@ namespace getfem {
     compcond = ga_substitute
       ("((mu)*xi-pos_part((mu)*xi+100*(fbound)/(mu)))", dict);
     von_mises = ga_substitute
-      ("sqrt(3/2)*sqrt(Norm_sqr((2*(mu))*(Dev_En)-(2*(mu)+(Hk))*(Epn))"
-       "+sqr(2*(mu)*Trace(En)/3-(2*(mu)+(Hk))*Trace(Epn)))", dict);
+      ("sqrt(3/2)*sqrt(Norm_sqr((2*(mu))*(Dev_En)-(2*(mu))*(Epn))"
+       "+sqr(2*(mu)*Trace(En)/3-(2*(mu))*Trace(Epn)))", dict);
   }
 
 
@@ -840,8 +842,8 @@ namespace getfem {
     xi_np1 = ga_substitute
       ("pos_part(sqrt(3/2)*Norm(B)/(sigma_y)-1/(2*(mu)))/((theta)*(dt))", dict);
     von_mises = ga_substitute
-      ("sqrt(3/2)*sqrt(Norm_sqr((2*(mu))*(Dev_En)-(2*(mu)+(Hk))*(Epn))"
-       "+sqr(2*(mu)*Trace(En)/3-(2*(mu)+(Hk))*Trace(Epn)))", dict);
+      ("sqrt(3/2)*sqrt(Norm_sqr((2*(mu))*(Dev_En)-(2*(mu))*(Epn))"
+       "+sqr(2*(mu)*Trace(En)/3-(2*(mu))*Trace(Epn)))", dict);
   }
 
   // Assembly strings for isotropic elastoplasticity with Von-Mises
@@ -893,22 +895,21 @@ namespace getfem {
     dict["En"] = ga_substitute("Sym(Grad_Previous_u)", dict);
     dict["zetan"] = ga_substitute
       ("((Epn)+(1-(theta))*((dt)*(Previous_xi))*((2*(mu))*Deviator(En)"
-       "-(2*(mu)+(Hk))*(Epn)))", dict);
+       "-(2*(mu)+2/3*(Hk))*(Epn)))", dict);
     dict["etan"] = ga_substitute
       ("((alphan)+sqrt(2/3)*(1-(theta))*((dt)*(Previous_xi))*"
-       "Norm((2*(mu))*Deviator(En)-(2*(mu)+(Hk))*(Epn)))", dict);
-    dict["B"] = ga_substitute("((2*(mu))*Deviator(Enp1)-(2*(mu)+(Hk))*(zetan))",
-                              dict);
-    dict["beta"] =
-      ga_substitute("((theta)*(dt)*(xi)/(1+(2*(mu)+(Hk))*(theta)*(dt)*(xi)))",
-                    dict);
+       "Norm((2*(mu))*Deviator(En)-(2*(mu)+2/3*(Hk))*(Epn)))", dict);
+    dict["B"] = ga_substitute
+      ("((2*(mu))*Deviator(Enp1)-(2*(mu)+2/3*(Hk))*(zetan))", dict);
+    dict["beta"] = ga_substitute
+      ("((theta)*(dt)*(xi)/(1+(2*(mu)+2/3*(Hk))*(theta)*(dt)*(xi)))", dict);
     dict["Epnp1"] = Epnp1 = ga_substitute("((zetan)+(beta)*(B))", dict);
     alphanp1 = ga_substitute("((etan)+sqrt(2/3)*(beta)*Norm(B))", dict);
     dict["alphanp1"] = alphanp1;
     sigma_np1 = ga_substitute
       ("((lambda)*Trace(Enp1)*Id(meshdim)+2*(mu)*((Enp1)-(Epnp1)))", dict);
     dict["fbound"] = ga_substitute
-      ("(Norm((2*(mu))*Deviator(Enp1)-(2*(mu)+(Hk))*(Epnp1))"
+      ("(Norm((2*(mu))*Deviator(Enp1)-(2*(mu)+2/3*(Hk))*(Epnp1))"
        "-sqrt(2/3)*(sigma_y+(Hi)*(alphanp1)))",  dict);
     dict["sigma_after"] = sigma_after = ga_substitute
       ("((lambda)*Trace(Enp1)*Id(meshdim)+2*(mu)*((Enp1)-(Epn)))", dict);
@@ -969,15 +970,15 @@ namespace getfem {
     dict["En"] = ga_substitute("Sym(Grad_Previous_u)", dict);
     dict["zetan"] = ga_substitute
       ("((Epn)+(1-(theta))*((dt)*(Previous_xi))*((2*(mu))*Deviator(En)"
-       "-(2*(mu)+(Hk))*(Epn)))", dict);
+       "-(2*(mu)+2/3*(Hk))*(Epn)))", dict);
     dict["etan"] = ga_substitute
       ("((alphan)+sqrt(2/3)*(1-(theta))*((dt)*(Previous_xi))*"
-       "Norm((2*(mu))*Deviator(En)-(2*(mu)+(Hk))*(Epn)))", dict);
+       "Norm((2*(mu))*Deviator(En)-(2*(mu)+2/3*(Hk))*(Epn)))", dict);
 
-    dict["B"] = ga_substitute("((2*(mu))*Deviator(Enp1)-(2*(mu)+(Hk))*(zetan))",
-                                  dict);
+    dict["B"] = ga_substitute
+      ("((2*(mu))*Deviator(Enp1)-(2*(mu)+2/3*(Hk))*(zetan))", dict);
     dict["beta"] =
-      ga_substitute("(1/((Norm(B)+1e-40)*(2*(mu)+(Hk)+(2/3)*(Hi))))"
+      ga_substitute("(1/((Norm(B)+1e-40)*(2*(mu)+2/3*(Hk)+(2/3)*(Hi))))"
       "*pos_part(Norm(B)-sqrt(2/3)*((sigma_y)+(Hi)*(etan)))", dict);
     dict["Epnp1"] = Epnp1 = ga_substitute("((zetan)+(beta)*(B))", dict);
     alphanp1 = ga_substitute("((etan)+sqrt(2/3)*(beta)*Norm(B))", dict);
@@ -988,7 +989,7 @@ namespace getfem {
     dict["sigma_after"] = sigma_after = ga_substitute
       ("(lambda)*Trace(Enp1)*Id(meshdim)+2*(mu)*((Enp1)-(Epn))", dict);
     xi_np1 = ga_substitute
-      ("(((beta)/(1-(2*(mu)+(Hk))*(beta)))/((theta)*(dt)))", dict);
+      ("(((beta)/(1-(2*(mu)+2/3*(Hk))*(beta)))/((theta)*(dt)))", dict);
     von_mises = ga_substitute
       ("sqrt(3/2)*Norm(Deviator(sigma_after))", dict);
   }
@@ -1046,35 +1047,34 @@ namespace getfem {
     dict["Dev_Enp1"]= ga_substitute("(Enp1-(Trace(Enp1)/3)*Id(meshdim))", dict);
     dict["zetan"] = ga_substitute
       ("((Epn)+(1-(theta))*((dt)*(Previous_xi))*((2*(mu))*(Dev_En)"
-       "-(2*(mu)+(Hk))*(Epn)))", dict);
+       "-(2*(mu)+2/3*(Hk))*(Epn)))", dict);
     dict["etan"] = ga_substitute
       ("((alphan)+sqrt(2/3)*(1-(theta))*((dt)*(Previous_xi))*"
-       "sqrt(Norm_sqr((2*(mu))*(Dev_En)-(2*(mu)+(Hk))*(Epn))"
-       "+sqr(2*(mu)*Trace(En)/3-(2*(mu)+(Hk))*Trace(Epn))))", dict);
-    dict["B"] = ga_substitute("((2*(mu))*(Dev_Enp1)-(2*(mu)+(Hk))*(zetan))",
+       "sqrt(Norm_sqr((2*(mu))*(Dev_En)-(2*(mu)+2/3*(Hk))*(Epn))"
+       "+sqr(2*(mu)*Trace(En)/3-(2*(mu)+2/3*(Hk))*Trace(Epn))))", dict);
+    dict["B"] = ga_substitute("((2*(mu))*(Dev_Enp1)-(2*(mu)+2/3*(Hk))*(zetan))",
                               dict);
     dict["Norm_B"] = ga_substitute("sqrt(Norm_sqr(B)+sqr(2*(mu)*Trace(Enp1)/3"
-                                   "-(2*(mu)+(Hk))*Trace(zetan)))", dict);
+                                   "-(2*(mu)+2/3*(Hk))*Trace(zetan)))", dict);
 
-    dict["beta"] =
-      ga_substitute("((theta)*(dt)*(xi)/(1+(2*(mu)+(Hk))*(theta)*(dt)*(xi)))",
-                    dict);
+    dict["beta"] = ga_substitute
+      ("((theta)*(dt)*(xi)/(1+(2*(mu)+2/3*(Hk))*(theta)*(dt)*(xi)))", dict);
     dict["Epnp1"] = Epnp1 = ga_substitute("((zetan)+(beta)*(B))", dict);
     alphanp1 = ga_substitute("((etan)+sqrt(2/3)*(beta)*(Norm_B))", dict);
     dict["alphanp1"] = alphanp1;
     sigma_np1 = ga_substitute
       ("((lambda)*Trace(Enp1)*Id(meshdim)+2*(mu)*((Enp1)-(Epnp1)))", dict);
     dict["fbound"] = ga_substitute
-      ("(sqrt(Norm_sqr((2*(mu))*(Dev_Enp1)-(2*(mu)+(Hk))*(Epnp1))"
-       "+sqr(2*(mu)*Trace(Enp1)/3-(2*(mu)+(Hk))*Trace(Epnp1)))"
+      ("(sqrt(Norm_sqr((2*(mu))*(Dev_Enp1)-(2*(mu)+2/3*(Hk))*(Epnp1))"
+       "+sqr(2*(mu)*Trace(Enp1)/3-(2*(mu)+2/3*(Hk))*Trace(Epnp1)))"
        "-sqrt(2/3)*(sigma_y+(Hi)*(alphanp1)))",  dict);
     sigma_after = ga_substitute
       ("((lambda)*Trace(Enp1)*Id(meshdim)+2*(mu)*((Enp1)-(Epn)))", dict);
     compcond = ga_substitute
       ("((mu)*xi-pos_part((mu)*xi+100*(fbound)/(mu)))", dict);
     von_mises = ga_substitute
-      ("sqrt(3/2)*sqrt(Norm_sqr((2*(mu))*(Dev_En)-(2*(mu)+(Hk))*(Epn))"
-       "+sqr(2*(mu)*Trace(En)/3-(2*(mu)+(Hk))*Trace(Epn)))", dict);
+      ("sqrt(3/2)*sqrt(Norm_sqr((2*(mu))*(Dev_En)-(2*(mu)+2/3*(Hk))*(Epn))"
+       "+sqr(2*(mu)*Trace(En)/3-(2*(mu)+2/3*(Hk))*Trace(Epn)))", dict);
   }
 
   // Assembly strings for isotropic elastoplasticity with Von-Mises
@@ -1133,18 +1133,18 @@ namespace getfem {
     dict["Dev_Enp1"]= ga_substitute("(Enp1-(Trace(Enp1)/3)*Id(meshdim))", dict);
     dict["zetan"] = ga_substitute
       ("((Epn)+(1-(theta))*((dt)*(Previous_xi))*((2*(mu))*(Dev_En)"
-       "-(2*(mu)+(Hk))*(Epn)))", dict);
+       "-(2*(mu)+2/3*(Hk))*(Epn)))", dict);
     dict["etan"] = ga_substitute
       ("((alphan)+sqrt(2/3)*(1-(theta))*((dt)*(Previous_xi))*"
-       "sqrt(Norm_sqr((2*(mu))*(Dev_En)-(2*(mu)+(Hk))*(Epn))"
-       "+sqr(2*(mu)*Trace(En)/3-(2*(mu)+(Hk))*Trace(Epn))))", dict);
+       "sqrt(Norm_sqr((2*(mu))*(Dev_En)-(2*(mu)+2/3*(Hk))*(Epn))"
+       "+sqr(2*(mu)*Trace(En)/3-(2*(mu)+2/3*(Hk))*Trace(Epn))))", dict);
 
-    dict["B"] = ga_substitute("((2*(mu))*(Dev_Enp1)-(2*(mu)+(Hk))*(zetan))",
-                                  dict);
+    dict["B"] = ga_substitute
+      ("((2*(mu))*(Dev_Enp1)-(2*(mu)+2/3*(Hk))*(zetan))", dict);
     dict["Norm_B"] = ga_substitute("sqrt(Norm_sqr(B)+sqr(2*(mu)*Trace(Enp1)/3"
-                                   "-(2*(mu)+(Hk))*Trace(zetan)))", dict);
+                                   "-(2*(mu)+2/3*(Hk))*Trace(zetan)))", dict);
     dict["beta"] =
-      ga_substitute("(1/(((Norm_B)+1e-40)*(2*(mu)+(Hk)+(2/3)*(Hi))))"
+      ga_substitute("(1/(((Norm_B)+1e-40)*(2*(mu)+2/3*(Hk)+(2/3)*(Hi))))"
       "*pos_part((Norm_B)-sqrt(2/3)*((sigma_y)+(Hi)*(etan)))", dict);
     dict["Epnp1"] = Epnp1 = ga_substitute("((zetan)+(beta)*(B))", dict);
     alphanp1 = ga_substitute("((etan)+sqrt(2/3)*(beta)*(Norm_B))", dict);
@@ -1155,10 +1155,10 @@ namespace getfem {
     sigma_after = ga_substitute
       ("(lambda)*Trace(Enp1)*Id(meshdim)+2*(mu)*((Enp1)-(Epn))", dict);
     xi_np1 = ga_substitute
-      ("(((beta)/(1-(2*(mu)+(Hk))*(beta)))/((theta)*(dt)))", dict);
+      ("(((beta)/(1-(2*(mu)+2/3*(Hk))*(beta)))/((theta)*(dt)))", dict);
     von_mises = ga_substitute
-      ("sqrt(3/2)*sqrt(Norm_sqr((2*(mu))*(Dev_En)-(2*(mu)+(Hk))*(Epn))"
-       "+sqr(2*(mu)*Trace(En)/3-(2*(mu)+(Hk))*Trace(Epn)))", dict);
+      ("sqrt(3/2)*sqrt(Norm_sqr((2*(mu))*(Dev_En)-(2*(mu)+2/3*(Hk))*(Epn))"
+       "+sqr(2*(mu)*Trace(En)/3-(2*(mu)+2/3*(Hk))*Trace(Epn)))", dict);
   }
 
   void build_isotropic_perfect_elastoplasticity_expressions_generic
@@ -1546,22 +1546,21 @@ namespace getfem {
       _DEVLOGBETR_3D_ =
       _DEVLOGBETR_ = "(Deviator(Logm("+_F_+"*"+_INVCP0_+"*"+_F_+"')))";
     }
-    const std::string _DEVLOGBE_("((1-2*"+_KSI_+")*"+_DEVLOGBETR_+")");
-    const std::string _DEVLOGBE_3D_("((1-2*"+_KSI_+")*"+_DEVLOGBETR_3D_+")");
-    const std::string _DEVTAU_("("+G+")*pow("+_J_+",-"+_TWOTHIRD_+")*"+_DEVLOGBE_);
+    const std::string _DEVTAUTR_("("+G+"*"+_DEVLOGBETR_+")");
+    const std::string _DEVTAUTR_3D_("("+G+"*"+_DEVLOGBETR_3D_+")");
+    const std::string _DEVTAU_("((1-2*"+_KSI_+")*"+_DEVTAUTR_+")");
+    const std::string _DEVTAU_3D_("((1-2*"+_KSI_+")*"+_DEVTAUTR_3D_+")");
     const std::string _TAU_("("+_P_+"*"+_I_+"+"+_DEVTAU_+")");
 
     const std::string _PLASTSTRAIN_("("+plaststrain0+"+"+_KSI_+"*Norm"+_DEVLOGBETR_3D_+")");
     const std::string _SIGMA_Y_(sigma_y+"("+_PLASTSTRAIN_+")");
-    const std::string _DEVSIGMA_("("+G+"*pow("+_J_+",-"+_FIVETHIRD_+")*"+_DEVLOGBE_3D_+")");
-    const std::string _DEVSIGMATR_("("+G+"*pow("+_J_+",-"+_FIVETHIRD_+")*"+_DEVLOGBETR_3D_+")");
 
     // results
     expr = _TAU_+":(Grad_Test_"+_U_+"*Inv"+_F_+")";
     if (mixed)
       expr += "+("+pressname+"/("+K+")+log("+_J_+")/"+_J_+")*Test_"+pressname;
-    expr += "+(Norm"+_DEVSIGMA_+
-            "-min("+_SIGMA_Y_+",Norm"+_DEVSIGMATR_+"+1e-12*"+_KSI_+"))*Test_"+_KSI_;
+    expr += "+(Norm"+_DEVTAU_+
+            "-min("+_SIGMA_Y_+",Norm"+_DEVTAUTR_+"+1e-12*"+_KSI_+"))*Test_"+_KSI_;
 
     plaststrain = _PLASTSTRAIN_;
 
@@ -1574,7 +1573,7 @@ namespace getfem {
     invCp += ":((Inv"+_F3d_+"*Expm(-"+_KSI_+"*"+_DEVLOGBETR_3D_+")*"+_F3d_+")*"+_INVCP0_+
               "*(Inv"+_F3d_+"*Expm(-"+_KSI_+"*"+_DEVLOGBETR_3D_+")*"+_F3d_+")')";
 
-    vm = _SQRTTHREEHALF_+"*Norm("+_DEVSIGMA_+")";
+    vm = _SQRTTHREEHALF_+"*Norm("+_DEVTAU_+")/"+_J_;
   }
 
   size_type add_finite_strain_elastoplasticity_brick
