@@ -923,22 +923,21 @@ namespace gmm {
   template <typename IT, typename ORG, typename VECT> inline
   void set_to_end(IT &, ORG, const VECT *, linalg_const) { }
 
-
   template <typename IT, typename ORG, typename VECT> inline
   void set_to_begin(IT &, ORG, VECT *v, linalg_modifiable)
-  { GMM_ASSERT3(!is_sparse(*v), "internal_error"); v = 0; }
+  { GMM_ASSERT3(!is_sparse(*v), "internal_error"); (void)v; }
 
   template <typename IT, typename ORG, typename VECT> inline
   void set_to_begin(IT &, ORG, const VECT *v, linalg_modifiable)
-  { GMM_ASSERT3(!is_sparse(*v), "internal_error"); v = 0; }
+  { GMM_ASSERT3(!is_sparse(*v), "internal_error"); (void)v; }
  
   template <typename IT, typename ORG, typename VECT> inline
   void set_to_end(IT &, ORG, VECT *v, linalg_modifiable)
-  { GMM_ASSERT3(!is_sparse(*v), "internal_error"); v = 0; }
+  { GMM_ASSERT3(!is_sparse(*v), "internal_error"); (void)v; }
   
   template <typename IT, typename ORG, typename VECT> inline
   void set_to_end(IT &, ORG, const VECT *v, linalg_modifiable)
-  { GMM_ASSERT3(!is_sparse(*v), "internal_error"); v = 0; }
+  { GMM_ASSERT3(!is_sparse(*v), "internal_error"); (void)v; }
 
   /* ******************************************************************** */
   /*		General index for certain algorithms.         		  */
