@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2002-2015 Yves Renard, Julien Pommier.
+ Copyright (C) 2002-2017 Yves Renard, Julien Pommier.
 
  This file is a part of GetFEM++
 
@@ -42,6 +42,8 @@
 
 using std::endl; using std::cout; using std::cerr;
 using std::ends; using std::cin;
+template <typename T> std::ostream &operator <<
+  (std::ostream &o, const std::vector<T>& m) { gmm::write(o,m); return o; }
 
 /* some GetFEM++ types that we will be using */
 using bgeot::base_small_vector; /* special class for small (dim<16) vectors */
@@ -229,13 +231,13 @@ void sol_ref_infinite_plane(scalar_type nu, scalar_type E, scalar_type sigma,
 				+9*cost*s2*mu*c2*c2)/(lambda-3*mu);
     }
   } else GMM_ASSERT1(false, "Unvalid mode");
-  if (!finite(U[0]))
+  if (!std::isfinite(U[0]))
     cerr << "raaah not a number ... nu=" << nu << ", E=" << E << ", sig="
 	 << sigma << ", a=" << a << ", xx=" << xx << ", y=" << y << ", r="
 	 << r << ", sqrtr=" << sqrtr << ", cost=" << cost << ", U=" << U[0]
 	 << "," << U[1] << endl;
-  assert(finite(U[0]));
-  assert(finite(U[1]));
+  assert(std::isfinite(U[0]));
+  assert(std::isfinite(U[1]));
 }
 
 struct exact_solution {
@@ -248,15 +250,14 @@ struct exact_solution {
 	    getfem::level_set &ls) {
     std::vector<getfem::pglobal_function> cfun(4);
     for (unsigned j=0; j < 4; ++j) {
-      getfem::crack_singular_xy_function *s = 
-	new getfem::crack_singular_xy_function(j);
-      cfun[j] = getfem::global_function_on_level_set(ls, *s);
+      auto s = std::make_shared<getfem::crack_singular_xy_function>(j);
+      cfun[j] = getfem::global_function_on_level_set(ls, s);
     }
 
     mf.set_functions(cfun);
     
     mf.set_qdim(1);
-    
+
     U.resize(8); assert(mf.nb_dof() == 4);
     getfem::base_vector::iterator it = U.begin();
     scalar_type coeff=0.;
@@ -550,7 +551,6 @@ base_small_vector ls_function(const base_node P, int num = 0) {
 bool crack_problem::solve(plain_vector &U) {
   size_type N = mesh.dim();
   ls.reinit();  
-  cout << "ls.get_mesh_fem().nb_dof() = " << ls.get_mesh_fem().nb_dof() << "\n";
   for (size_type d = 0; d < ls.get_mesh_fem().nb_basic_dof(); ++d) {
     ls.values(0)[d] = ls_function(ls.get_mesh_fem().point_of_basic_dof(d), 0)[0];
     ls.values(1)[d] = ls_function(ls.get_mesh_fem().point_of_basic_dof(d), 0)[1];
@@ -582,20 +582,18 @@ bool crack_problem::solve(plain_vector &U) {
   std::vector<getfem::pglobal_function> vfunc(4);
   for (size_type i = 0; i < vfunc.size(); ++i) {
     /* use the singularity */
-    getfem::abstract_xy_function *s = 
-      new getfem::crack_singular_xy_function(unsigned(i));
+    getfem::pxy_function
+      s = std::make_shared<getfem::crack_singular_xy_function>(unsigned(i));
     if (enrichment_option != FIXED_ZONE && 
 	enrichment_option != GLOBAL_WITH_MORTAR) {
       /* use the product of the singularity function
 	 with a cutoff */
-      getfem::abstract_xy_function *c = 
-	new getfem::cutoff_xy_function(int(cutoff.fun_num),
-				       cutoff.radius, 
-				       cutoff.radius1,
-				       cutoff.radius0);
-      s = new getfem::product_of_xy_functions(*s, *c);
+      getfem::pxy_function c
+	= std::make_shared<getfem::cutoff_xy_function>
+	(int(cutoff.fun_num), cutoff.radius, cutoff.radius1, cutoff.radius0);
+      s = std::make_shared<getfem::product_of_xy_functions>(s, c);
     }
-    vfunc[i] = getfem::global_function_on_level_set(ls, *s);
+    vfunc[i] = getfem::global_function_on_level_set(ls, s);
   }
   
   mf_sing_u.set_functions(vfunc);
@@ -794,6 +792,7 @@ bool crack_problem::solve(plain_vector &U) {
     GMM_ASSERT1(!mf_mortar.is_reduced(), "To be adapted");
     sparse_matrix M(mf_mortar.nb_dof(), mf_mortar.nb_dof());
     getfem::asm_mass_matrix(M, mim, mf_mortar, MORTAR_BOUNDARY_OUT);
+    
     for (dal::bv_visitor_c d(mf_mortar.basic_dof_on_region(MORTAR_BOUNDARY_OUT)); 
 	 !d.finished(); ++d) {
       if (M(d,d) > 1e-8) ind_mortar.push_back(d);
@@ -812,14 +811,13 @@ bool crack_problem::solve(plain_vector &U) {
     /* build the mortar constraint matrix -- note that the integration
        method is conformal to the crack
      */
-    getfem::asm_mass_matrix(H0, mim, mf_mortar, mf_u(), 
-			    MORTAR_BOUNDARY_OUT);
+    getfem::asm_mass_matrix(H0, mim, mf_mortar, mf_u(), MORTAR_BOUNDARY_OUT);   
     gmm::copy(gmm::sub_matrix(H0, sub_i, sub_j), H);
 
     gmm::clear(H0);
     getfem::asm_mass_matrix(H0, mim, mf_mortar, mf_u(), 
 			    MORTAR_BOUNDARY_IN);
-    gmm::add(gmm::scaled(gmm::sub_matrix(H0, sub_i, sub_j), -1), H);
+    gmm::add(gmm::scaled(gmm::sub_matrix(H0, sub_i, sub_j), -1.0), H);
 
 
     /* because of the discontinuous partition of mf_u(), some levelset
