@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2002-2016 Yves Renard, Julien Pommier.
+ Copyright (C) 2002-2017 Yves Renard, Julien Pommier.
 
  This file is a part of GetFEM++
 
@@ -27,7 +27,8 @@
 #include <map>
 using std::endl; using std::cout; using std::cerr;
 using std::ends; using std::cin;
-
+template <typename T> std::ostream &operator <<
+  (std::ostream &o, const std::vector<T>& m) { gmm::write(o,m); return o; }
 
 using getfem::size_type;
 using getfem::short_type;
@@ -455,13 +456,11 @@ int main(/* int argc, char **argv */) {
     getfem::pfem pf = getfem::QK_fem(2,1); //getfem::classical_fem(bgeot::parallelepiped_linear_geotrans(2),1);
     return 100;*/
 
-
-
     int ok = 0;
     getfem::pintegration_method im_none = getfem::int_method_descriptor("IM_NONE()");
     try {
       cout << "nbpts=" << im_none->structure()->nb_points() << "\n";
-    } catch (gmm::gmm_error e) {
+    } catch (const gmm::gmm_error &e) {
       ok = 1;
     }
     GMM_ASSERT1(ok, "IM_NONE failed");
