@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2007-2016 Yves Renard, Julien Pommier.
+ Copyright (C) 2007-2017 Yves Renard, Julien Pommier.
 
  This file is a part of GetFEM++
 
@@ -98,6 +98,7 @@ namespace test {
       node_id(gmm::uint32_type b, gmm::uint32_type cid, bool al_) : p((b*BLOCKSZ+cid)+(al_?0x80000000:0)) {}
       bool null() const { return p == 0; }
       void nullify() { p = 0; }
+      bool operator ==(const node_id &a) { return p == a.p; }
     };
 
 
@@ -270,7 +271,7 @@ namespace test {
       : id(allocate(a.size())) { std::transform(a.begin(), a.end(), begin(), op); }
     template<class BINOP> small_vector(const small_vector<T>& a, const small_vector<T>& b, BINOP op) 
       : id(allocate(a.size())) { std::transform(a.begin(), a.end(), b.begin(), begin(), op); }
-    bool empty() const { return this->id==node_id(0); }
+    bool empty() const { return this->id == node_id(0); }
     unsigned char refcnt() const { return allocator().refcnt(id); }
     dim_type size() const { return allocator().obj_sz(id)/sizeof(value_type); }
     small_vector<T> operator+(const small_vector<T>& other) const 
@@ -457,7 +458,7 @@ namespace getfem {
     typedef double *iterator;
     typedef const double * const_iterator;
     pointer p;
-    pointer base() const { return (double*)((unsigned long)p&(~7UL)); }
+    pointer base() const { return (double*)((size_type)p&(~7UL)); }
     reference operator[](size_type i) { return base()[i]; }
     const double& operator[](size_type i) const { return base()[i]; }
     micro_vec() : p(0) {}
@@ -490,7 +491,7 @@ namespace getfem {
       return *this;
     }
     ~micro_vec() { deallocate(); }
-    size_type size() const { return (unsigned long)p & 7UL; }
+    size_type size() const { return (size_type)(p) & 7UL; }
     micro_vec(const micro_vec& va, const micro_vec& vb) : p(alloc(va.size())) { 
       /*double * restrict__ pb = base(), *pe = base()+size(), *pva = va.base(), *pvb = vb.base();
 	for (; pb < pe;) *pb++=*pva+++*pvb++;*/
@@ -550,10 +551,11 @@ namespace getfem {
       for (size_type j=0; j < vv.size(); ++j) {
 	for (size_type k=0; k < vv[j].size(); ++k) {
 	  vv[j][k] = double(j+k);
-	  assert(vv[j][k] == j+k);
+	  assert(vv[j][k] == double(j+k));
 	}
 	vv[j] = vv[j];
-	for (size_type k=0; k < vv[j].size(); ++k) assert(vv[j][k] == j+k);
+	for (size_type k=0; k < vv[j].size(); ++k)
+	  assert(vv[j][k] == double(j+k));
       }
     }
     cout << "remplissage : " << c.toc().cpu() << " sec\n";
